@@ -104,6 +104,7 @@ for CC in ["gcc", "clang"]:
     for ssl in [
         "stock",
         "OPENSSL_VERSION=1.0.2u",
+        "OPENSSL_VERSION=3.0.1",
         "LIBRESSL_VERSION=2.9.2",
         "LIBRESSL_VERSION=3.3.3",
 #        "BORINGSSL=yes",
@@ -111,6 +112,8 @@ for CC in ["gcc", "clang"]:
         flags = ["USE_OPENSSL=1"]
         if ssl == "BORINGSSL=yes":
             flags.append("USE_QUIC=1")
+        if "OPENSSL_VERSION=3.0." in ssl:
+            flags.append('DEBUG_CFLAGS="-g -Wno-deprecated-declarations"')
         if ssl != "stock":
             flags.append("SSL_LIB=${HOME}/opt/lib")
             flags.append("SSL_INC=${HOME}/opt/include")
