@@ -156,15 +156,28 @@ static void flt_ot_return_void(const struct filter *f, char **err)
 static int flt_ot_init(struct proxy *p, struct flt_conf *fconf)
 {
 	struct flt_ot_conf *conf = FLT_OT_DEREF(fconf, conf, NULL);
-	int                 retval = FLT_OT_RET_OK;
+	char               *err = NULL;
+	int                 retval = FLT_OT_RET_ERROR;
 
 	FLT_OT_FUNC("%p, %p", p, fconf);
 
 	if (conf == NULL)
-		FLT_OT_RETURN(FLT_OT_RET_ERROR);
+		FLT_OT_RETURN(retval);
 
 	flt_ot_cli_init();
 
+	/*
+	 * Initialize the OpenTracing library.
+	 */
+	retval = ot_init(&(conf->tracer->tracer), conf->tracer->plugin, &err);
+	if (retval != FLT_OT_RET_ERROR)
+		/* Do nothing. */;
+	else if (err != NULL) {
+		FLT_OT_ALERT("%s", err);
+
+		FLT_OT_ERR_FREE(err);
+	}
+
 	FLT_OT_RETURN(retval);
 }
 
@@ -440,11 +453,11 @@ static int flt_ot_init_per_thread(struct proxy *p, struct flt_conf *fconf)
 		FLT_OT_RETURN(retval);
 
 	/*
-	 * Initialize the OpenTracing library.
+	 * Start the OpenTracing library tracer thread.
 	 * Enable HTX streams filtering.
 	 */
-	if (conf->tracer->tracer == NULL) {
-		retval = ot_init(&(conf->tracer->tracer), conf->tracer->config, conf->tracer->plugin, &err);
+	if (!(fconf->flags & FLT_CFG_FL_HTX)) {
+		retval = ot_start(conf->tracer->tracer, conf->tracer->cfgbuf, &err);
 		if (retval != FLT_OT_RET_ERROR)
 			fconf->flags |= FLT_CFG_FL_HTX;
 		else if (err != NULL) {
