@@ -83,6 +83,13 @@ enum {
 	ARGC_USRV,     /* use-server message */
 	ARGC_HERR,     /* http-error */
 	ARGC_OT,       /* opentracing scope args */
+	ARGC_TCO,      /* tcp-request connection expression */
+	ARGC_TSE,      /* tcp-request session expression */
+	ARGC_TRQ,      /* tcp-request content expression */
+	ARGC_TRS,      /* tcp-response content expression */
+	ARGC_TCK,      /* tcp-check expression */
+	ARGC_CFG,      /* configuration expression */
+	ARGC_CLI,      /* CLI expression*/
 };
 
 /* flags used when compiling and executing regex */
