@@ -267,6 +267,12 @@ typedef struct { } empty_t;
 #endif
 #endif
 
+/* dl_iterate_phdr() is available in GLIBC 2.2.4 and up. Let's round up to 2.3.x */
+#if defined(USE_DL) && defined(__GNU_LIBRARY__) && (__GLIBC__ > 2 || __GLIBC__ == 2 && __GLIBC_MINOR__ >= 3)
+#define HA_HAVE_DL_ITERATE_PHDR
+#define HA_HAVE_DUMP_LIBS
+#endif
+
 /* malloc_trim() can be very convenient to reclaim unused memory especially
  * from huge pattern files. It's available (and really usable) in glibc 2.8 and
  * above.
@@ -300,9 +306,10 @@ typedef struct { } empty_t;
 #endif
 
 /* Max number of file descriptors we send in one sendmsg(). Linux seems to be
- * able to send 253 fds per sendmsg(), not sure about the other OSes.
+ * able to send 253 fds per sendmsg(), however musl is limited to 252, not sure
+ * about the other OSes.
  */
-#define MAX_SEND_FD 253
+#define MAX_SEND_FD 252
 
 /* Make the new complex name for the xxhash function easier to remember
  * and use.
