@@ -86,6 +86,9 @@ enum {
 	 * the stream-interface :
 	 */
 	CS_FL_NOT_FIRST     = 0x00100000,  /* this stream is not the first one */
+
+	/* flags set by the mux relayed to the stream */
+	CS_FL_WEBSOCKET     = 0x00200000,  /* websocket stream */
 };
 
 /* cs_shutr() modes */
@@ -131,7 +134,7 @@ enum {
 	CO_FL_CTRL_READY    = 0x00000100, /* FD was registered, fd_delete() needed */
 	CO_FL_XPRT_READY    = 0x00000200, /* xprt_start() done, xprt can be used */
 
-	/* unused : 0x00000400 */
+	CO_FL_WANT_DRAIN    = 0x00000400, /* try to drain pending data when closing */
 
 	/* This flag is used by data layers to indicate they had to stop
 	 * receiving data because a buffer was full. The connection handler
@@ -271,10 +274,11 @@ enum {
 
 /* flags that can be passed to xprt->rcv_buf() and mux->rcv_buf() */
 enum {
-	CO_RFL_BUF_WET     = 0x0001,    /* Buffer still has some output data present */
-	CO_RFL_BUF_FLUSH   = 0x0002,    /* Flush mux's buffers but don't read more data */
-	CO_RFL_READ_ONCE   = 0x0004,    /* don't loop even if the request/response is small */
-	CO_RFL_KEEP_RECV   = 0x0008,    /* Instruct the mux to still wait for read events  */
+	CO_RFL_BUF_WET       = 0x0001,    /* Buffer still has some output data present */
+	CO_RFL_BUF_FLUSH     = 0x0002,    /* Flush mux's buffers but don't read more data */
+	CO_RFL_READ_ONCE     = 0x0004,    /* don't loop even if the request/response is small */
+	CO_RFL_KEEP_RECV     = 0x0008,    /* Instruct the mux to still wait for read events  */
+	CO_RFL_BUF_NOT_STUCK = 0x0010,    /* Buffer is not stuck. Optims are possible during data copy */
 };
 
 /* flags that can be passed to xprt->snd_buf() and mux->snd_buf() */
