@@ -145,6 +145,8 @@ static inline void conn_ctrl_init(struct connection *conn)
 static inline void conn_ctrl_close(struct connection *conn)
 {
 	if (!conn->xprt && (conn->flags & CO_FL_CTRL_READY)) {
+		if ((conn->flags & (CO_FL_WANT_DRAIN | CO_FL_SOCK_RD_SH)) == CO_FL_WANT_DRAIN)
+			conn_ctrl_drain(conn);
 		conn->flags &= ~CO_FL_CTRL_READY;
 		if (conn->ctrl->ctrl_close)
 			conn->ctrl->ctrl_close(conn);
@@ -247,6 +249,16 @@ static inline int conn_notify_mux(struct connection *conn, int old_flags, int fo
 	 *  - end of HANDSHAKE with CONNECTED set
 	 *  - regular data layer activity
 	 *
+	 * One tricky case is the wake up on read0 or error on an idle
+	 * backend connection, that can happen on a connection that is still
+	 * polled while at the same moment another thread is about to perform a
+	 * takeover. The solution against this is to remove the connection from
+	 * the idle list if it was in it, and possibly reinsert it at the end
+	 * if the connection remains valid. The cost is non-null (locked tree
+	 * removal) but remains low given that this is extremely rarely called.
+	 * In any case it's guaranteed by the FD's thread_mask that we're
+	 * called from the same thread the connection is queued in.
+	 *
 	 * Note that the wake callback is allowed to release the connection and
 	 * the fd (and return < 0 in this case).
 	 */
@@ -254,9 +266,28 @@ static inline int conn_notify_mux(struct connection *conn, int old_flags, int fo
 	     ((conn->flags ^ old_flags) & CO_FL_NOTIFY_DONE) ||
 	     ((old_flags & CO_FL_WAIT_XPRT) && !(conn->flags & CO_FL_WAIT_XPRT))) &&
 	    conn->mux && conn->mux->wake) {
+		uint conn_in_list = conn->flags & CO_FL_LIST_MASK;
+		struct server *srv = objt_server(conn->target);
+
+		if (conn_in_list) {
+			HA_SPIN_LOCK(IDLE_CONNS_LOCK, &idle_conns[tid].idle_conns_lock);
+			conn_delete_from_tree(&conn->hash_node->node);
+			HA_SPIN_UNLOCK(IDLE_CONNS_LOCK, &idle_conns[tid].idle_conns_lock);
+		}
+
 		ret = conn->mux->wake(conn);
 		if (ret < 0)
 			goto done;
+
+		if (conn_in_list) {
+			struct eb_root *root = (conn_in_list == CO_FL_SAFE_LIST) ?
+				&srv->per_thr[tid].safe_conns :
+				&srv->per_thr[tid].idle_conns;
+
+			HA_SPIN_LOCK(IDLE_CONNS_LOCK, &idle_conns[tid].idle_conns_lock);
+			ebmb_insert(root, &conn->hash_node->node, sizeof(conn->hash_node->hash));
+			HA_SPIN_UNLOCK(IDLE_CONNS_LOCK, &idle_conns[tid].idle_conns_lock);
+		}
 	}
  done:
 	return ret;
@@ -265,6 +296,8 @@ static inline int conn_notify_mux(struct connection *conn, int old_flags, int fo
 /* shut read */
 static inline void cs_shutr(struct conn_stream *cs, enum cs_shr_mode mode)
 {
+	if (cs->flags & CS_FL_SHR)
+		return;
 
 	/* clean data-layer shutdown */
 	if (cs->conn->mux && cs->conn->mux->shutr)
@@ -275,6 +308,8 @@ static inline void cs_shutr(struct conn_stream *cs, enum cs_shr_mode mode)
 /* shut write */
 static inline void cs_shutw(struct conn_stream *cs, enum cs_shw_mode mode)
 {
+	if (cs->flags & CS_FL_SHW)
+		return;
 
 	/* clean data-layer shutdown */
 	if (cs->conn->mux && cs->conn->mux->shutw)
@@ -287,7 +322,6 @@ static inline void cs_close(struct conn_stream *cs)
 {
 	cs_shutw(cs, CS_SHW_SILENT);
 	cs_shutr(cs, CS_SHR_RESET);
-	cs->flags = CS_FL_NONE;
 }
 
 /* completely close a conn_stream after draining possibly pending data (but do not detach it) */
@@ -295,7 +329,6 @@ static inline void cs_drain_and_close(struct conn_stream *cs)
 {
 	cs_shutw(cs, CS_SHW_SILENT);
 	cs_shutr(cs, CS_SHR_DRAIN);
-	cs->flags = CS_FL_NONE;
 }
 
 /* sets CS_FL_ERROR or CS_FL_ERR_PENDING on the cs */
@@ -775,6 +808,7 @@ static inline const char *conn_err_code_str(struct connection *c)
 
 	case CO_ER_CIP_EMPTY:     return "Connection closed while waiting for NetScaler Client IP header";
 	case CO_ER_CIP_ABORT:     return "Connection error while waiting for NetScaler Client IP header";
+	case CO_ER_CIP_TIMEOUT:   return "Timeout while waiting for a NetScaler Client IP header";
 	case CO_ER_CIP_TRUNCATED: return "Truncated NetScaler Client IP header received";
 	case CO_ER_CIP_BAD_MAGIC: return "Received an invalid NetScaler Client IP magic number";
 	case CO_ER_CIP_BAD_PROTO: return "Received an unhandled protocol in the NetScaler Client IP header";
@@ -793,6 +827,7 @@ static inline const char *conn_err_code_str(struct connection *c)
 	case CO_ER_SSL_HANDSHAKE_HB: return "SSL handshake failure after heartbeat";
 	case CO_ER_SSL_KILLED_HB: return "Stopped a TLSv1 heartbeat attack (CVE-2014-0160)";
 	case CO_ER_SSL_NO_TARGET: return "Attempt to use SSL on an unknown target (internal error)";
+	case CO_ER_SSL_EARLY_FAILED: return "Server refused early data";
 
 	case CO_ER_SOCKS4_SEND:    return "SOCKS4 Proxy write error during handshake";
 	case CO_ER_SOCKS4_RECV:    return "SOCKS4 Proxy read error during handshake";
@@ -802,6 +837,8 @@ static inline const char *conn_err_code_str(struct connection *c)
 	return NULL;
 }
 
+int conn_update_alpn(struct connection *conn, const struct ist alpn, int force);
+
 static inline const char *conn_get_ctrl_name(const struct connection *conn)
 {
 	if (!conn || !conn_ctrl_ready(conn))
@@ -1033,7 +1070,8 @@ static inline int conn_install_mux_fe(struct connection *conn, void *ctx)
  * <ctx>. If the mux protocol is forced, we use it to find the best mux. Returns
  * < 0 on error.
  */
-static inline int conn_install_mux_be(struct connection *conn, void *ctx, struct session *sess)
+static inline int conn_install_mux_be(struct connection *conn, void *ctx, struct session *sess,
+                                      const struct mux_ops *force_mux_ops)
 {
 	struct server *srv = objt_server(conn->target);
 	struct proxy  *prx = objt_proxy(conn->target);
@@ -1045,8 +1083,12 @@ static inline int conn_install_mux_be(struct connection *conn, void *ctx, struct
 	if (!prx) // target must be either proxy or server
 		return -1;
 
-	if (srv && srv->mux_proto)
+	if (srv && srv->mux_proto && likely(!force_mux_ops)) {
 		mux_ops = srv->mux_proto->mux;
+	}
+	else if (srv && unlikely(force_mux_ops)) {
+		mux_ops = force_mux_ops;
+	}
 	else {
 		struct ist mux_proto;
 		const char *alpn_str = NULL;
