@@ -39,17 +39,17 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 
 static inline void http_req_keywords_register(struct action_kw_list *kw_list)
 {
-	LIST_ADDQ(&http_req_keywords.list, &kw_list->list);
+	LIST_APPEND(&http_req_keywords.list, &kw_list->list);
 }
 
 static inline void http_res_keywords_register(struct action_kw_list *kw_list)
 {
-	LIST_ADDQ(&http_res_keywords.list, &kw_list->list);
+	LIST_APPEND(&http_res_keywords.list, &kw_list->list);
 }
 
 static inline void http_after_res_keywords_register(struct action_kw_list *kw_list)
 {
-	LIST_ADDQ(&http_after_res_keywords.list, &kw_list->list);
+	LIST_APPEND(&http_after_res_keywords.list, &kw_list->list);
 }
 
 struct action_kw *action_http_req_custom(const char *kw);
