@@ -27,7 +27,7 @@
 
 struct proxy;
 struct server;
-void chash_init_server_tree(struct proxy *p);
+int chash_init_server_tree(struct proxy *p);
 struct server *chash_get_next_server(struct proxy *p, struct server *srvtoavoid);
 struct server *chash_get_server_hash(struct proxy *p, unsigned int hash, const struct server *avoid);
 
