@@ -28,14 +28,12 @@
 #include <haproxy/listener-t.h>
 #include <haproxy/sample-t.h>
 
-int tcp_bind_socket(int fd, int flags, struct sockaddr_storage *local, struct sockaddr_storage *remote);
-int tcp_pause_listener(struct listener *l);
-int tcp_connect_server(struct connection *conn, int flags);
-int tcp_get_src(int fd, struct sockaddr *sa, socklen_t salen, int dir);
-int tcp_get_dst(int fd, struct sockaddr *sa, socklen_t salen, int dir);
+extern struct protocol proto_tcp4;
+extern struct protocol proto_tcp6;
 
-/* Export some samples. */
-int smp_fetch_src(const struct arg *args, struct sample *smp, const char *kw, void *private);
+int tcp_bind_socket(int fd, int flags, struct sockaddr_storage *local, struct sockaddr_storage *remote);
+int tcp_connect_server(struct connection *conn, int flags);
+int tcp_is_foreign(int fd, sa_family_t family);
 
 #endif /* _HAPROXY_PROTO_TCP_H */
 
