@@ -25,6 +25,7 @@
 #include <haproxy/api.h>
 #include <haproxy/applet-t.h>
 #include <haproxy/freq_ctr.h>
+#include <haproxy/list.h>
 #include <haproxy/listener-t.h>
 #include <haproxy/proxy-t.h>
 #include <haproxy/server-t.h>
@@ -47,8 +48,10 @@ int resume_proxy(struct proxy *p);
 void stop_proxy(struct proxy *p);
 int  stream_set_backend(struct stream *s, struct proxy *be);
 
+void free_proxy(struct proxy *p);
 const char *proxy_cap_str(int cap);
 const char *proxy_mode_str(int mode);
+const char *proxy_find_best_option(const char *word, const char **extra);
 void proxy_store_name(struct proxy *px);
 struct proxy *proxy_find_by_id(int id, int cap, int table);
 struct proxy *proxy_find_by_name(const char *name, int cap, int table);
@@ -56,6 +59,15 @@ struct proxy *proxy_find_best_match(int cap, const char *name, int id, int *diff
 struct server *findserver(const struct proxy *px, const char *name);
 int proxy_cfg_ensure_no_http(struct proxy *curproxy);
 void init_new_proxy(struct proxy *p);
+void proxy_preset_defaults(struct proxy *defproxy);
+void proxy_free_defaults(struct proxy *defproxy);
+void proxy_destroy_defaults(struct proxy *px);
+void proxy_destroy_all_defaults();
+struct proxy *alloc_new_proxy(const char *name, unsigned int cap,
+                              char **errmsg);
+struct proxy *parse_new_proxy(const char *name, unsigned int cap,
+                              const char *file, int linenum,
+                              const struct proxy *defproxy);
 int get_backend_server(const char *bk_name, const char *sv_name,
 		       struct proxy **bk, struct server **sv);
 void proxy_capture_error(struct proxy *proxy, int is_back,
@@ -110,9 +122,9 @@ static inline void proxy_reset_timeouts(struct proxy *proxy)
 /* increase the number of cumulated connections received on the designated frontend */
 static inline void proxy_inc_fe_conn_ctr(struct listener *l, struct proxy *fe)
 {
-	_HA_ATOMIC_ADD(&fe->fe_counters.cum_conn, 1);
+	_HA_ATOMIC_INC(&fe->fe_counters.cum_conn);
 	if (l && l->counters)
-		_HA_ATOMIC_ADD(&l->counters->cum_conn, 1);
+		_HA_ATOMIC_INC(&l->counters->cum_conn);
 	HA_ATOMIC_UPDATE_MAX(&fe->fe_counters.cps_max,
 			     update_freq_ctr(&fe->fe_conn_per_sec, 1));
 }
@@ -121,9 +133,9 @@ static inline void proxy_inc_fe_conn_ctr(struct listener *l, struct proxy *fe)
 static inline void proxy_inc_fe_sess_ctr(struct listener *l, struct proxy *fe)
 {
 
-	_HA_ATOMIC_ADD(&fe->fe_counters.cum_sess, 1);
+	_HA_ATOMIC_INC(&fe->fe_counters.cum_sess);
 	if (l && l->counters)
-		_HA_ATOMIC_ADD(&l->counters->cum_sess, 1);
+		_HA_ATOMIC_INC(&l->counters->cum_sess);
 	HA_ATOMIC_UPDATE_MAX(&fe->fe_counters.sps_max,
 			     update_freq_ctr(&fe->fe_sess_per_sec, 1));
 }
@@ -131,7 +143,7 @@ static inline void proxy_inc_fe_sess_ctr(struct listener *l, struct proxy *fe)
 /* increase the number of cumulated connections on the designated backend */
 static inline void proxy_inc_be_ctr(struct proxy *be)
 {
-	_HA_ATOMIC_ADD(&be->be_counters.cum_conn, 1);
+	_HA_ATOMIC_INC(&be->be_counters.cum_conn);
 	HA_ATOMIC_UPDATE_MAX(&be->be_counters.sps_max,
 			     update_freq_ctr(&be->be_sess_per_sec, 1));
 }
@@ -139,9 +151,9 @@ static inline void proxy_inc_be_ctr(struct proxy *be)
 /* increase the number of cumulated requests on the designated frontend */
 static inline void proxy_inc_fe_req_ctr(struct listener *l, struct proxy *fe)
 {
-	_HA_ATOMIC_ADD(&fe->fe_counters.p.http.cum_req, 1);
+	_HA_ATOMIC_INC(&fe->fe_counters.p.http.cum_req);
 	if (l && l->counters)
-		_HA_ATOMIC_ADD(&l->counters->p.http.cum_req, 1);
+		_HA_ATOMIC_INC(&l->counters->p.http.cum_req);
 	HA_ATOMIC_UPDATE_MAX(&fe->fe_counters.p.http.rps_max,
 			     update_freq_ctr(&fe->fe_req_per_sec, 1));
 }
@@ -154,6 +166,10 @@ static inline int l7_status_match(struct proxy *p, int status)
 		return 0;
 
 	switch (status) {
+	case 401:
+		return (p->retry_type & PR_RE_401);
+	case 403:
+		return (p->retry_type & PR_RE_403);
 	case 404:
 		return (p->retry_type & PR_RE_404);
 	case 408:
