@@ -41,6 +41,8 @@
 #define STAT_SHOW_FDESC 0x00001000      /* show the field descriptions when possible */
 #define STAT_SHMODULES  0x00002000      /* conf: show modules */
 #define STAT_HIDE_MAINT 0x00004000	/* hide maint/disabled servers */
+#define STAT_CONVDONE   0x00008000	/* conf: rules conversion done */
+#define STAT_USE_FLOAT  0x00010000      /* use floats where possible in the outputs */
 
 #define STAT_BOUND      0x00800000	/* bound statistics to selected proxies/types/services */
 #define STAT_STARTED    0x01000000	/* some output has occurred */
@@ -323,6 +325,12 @@ enum info_field {
 	INF_BYTES_OUT_RATE,
 	INF_DEBUG_COMMANDS_ISSUED,
 	INF_CUM_LOG_MSGS,
+	INF_BUILD_INFO,
+	INF_MEMMAX_BYTES,
+	INF_POOL_ALLOC_BYTES,
+	INF_POOL_USED_BYTES,
+	INF_START_TIME_SEC,
+	INF_TAINTED,
 
 	/* must always be the last one */
 	INF_TOTAL_FIELDS
@@ -435,6 +443,7 @@ enum stat_field {
 	ST_F_USED_CONN_CUR,
 	ST_F_NEED_CONN_EST,
 	ST_F_UWEIGHT,
+	ST_F_AGG_SRV_CHECK_STATUS,
 
 	/* must always be the last one */
 	ST_F_TOTAL_FIELDS
@@ -509,11 +518,15 @@ enum stats_domain_px_cap {
 	STATS_PX_CAP_MASK = 0xff
 };
 
+extern THREAD_LOCAL void *trash_counters;
+
 #define EXTRA_COUNTERS(name) \
 	struct extra_counters *name
 
 #define EXTRA_COUNTERS_GET(counters, mod) \
-	(void *)((counters)->data + (mod)->counters_off[(counters)->type])
+	(likely(counters) ? \
+		((void *)((counters)->data + (mod)->counters_off[(counters)->type])) : \
+		(trash_counters))
 
 #define EXTRA_COUNTERS_REGISTER(counters, ctype, alloc_failed_label) \
 	do {                                                         \
