@@ -102,7 +102,7 @@ static inline unsigned long long ha_get_pthread_id(unsigned int thr)
 
 static inline void ha_thread_relax(void)
 {
-#if _POSIX_PRIORITY_SCHEDULING
+#ifdef _POSIX_PRIORITY_SCHEDULING
 	sched_yield();
 #endif
 }
@@ -238,7 +238,7 @@ static inline unsigned long long ha_get_pthread_id(unsigned int thr)
 
 static inline void ha_thread_relax(void)
 {
-#if _POSIX_PRIORITY_SCHEDULING
+#ifdef _POSIX_PRIORITY_SCHEDULING
 	sched_yield();
 #else
 	pl_cpu_relax();
@@ -381,6 +381,7 @@ enum lock_label {
 	STK_SESS_LOCK,
 	APPLETS_LOCK,
 	PEER_LOCK,
+	SHCTX_LOCK,
 	SSL_LOCK,
 	SSL_GEN_CERTS_LOCK,
 	PATREF_LOCK,
@@ -433,6 +434,7 @@ static inline const char *lock_label(enum lock_label label)
 	case STK_SESS_LOCK:        return "STK_SESS";
 	case APPLETS_LOCK:         return "APPLETS";
 	case PEER_LOCK:            return "PEER";
+	case SHCTX_LOCK:           return "SHCTX";
 	case SSL_LOCK:             return "SSL";
 	case SSL_GEN_CERTS_LOCK:   return "SSL_GEN_CERTS";
 	case PATREF_LOCK:          return "PATREF";
