@@ -1635,16 +1635,15 @@ skip_reuse:
 #endif
 	}
 	else {
+		s->flags |= SF_SRV_REUSED;
+
 		/* Currently there seems to be no known cases of xprt ready
 		 * without the mux installed here.
 		 */
 		BUG_ON(!srv_conn->mux);
 
-		/* Only consider we're doing reuse if the connection was
-		 * ready.
-		 */
-		if (srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY)
-			s->flags |= SF_SRV_REUSED;
+		if (!(srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY))
+			s->flags |= SF_SRV_REUSED_ANTICIPATED;
 	}
 
 	/* flag for logging source ip/port */
@@ -1670,8 +1669,10 @@ skip_reuse:
 		return err;
 
 #ifdef USE_OPENSSL
-	if (smp_make_safe(sni_smp))
-		ssl_sock_set_servername(srv_conn, sni_smp->data.u.str.area);
+	if (!(s->flags & SF_SRV_REUSED)) {
+		if (smp_make_safe(sni_smp))
+			ssl_sock_set_servername(srv_conn, sni_smp->data.u.str.area);
+	}
 #endif /* USE_OPENSSL */
 
 	/* The CO_FL_SEND_PROXY flag may have been set by the connect method,
@@ -2236,7 +2237,7 @@ void back_handle_st_cer(struct stream *s)
 
 	/* we probably have to release last stream from the server */
 	if (objt_server(s->target)) {
-		health_adjust(objt_server(s->target), HANA_STATUS_L4_ERR);
+		health_adjust(__objt_server(s->target), HANA_STATUS_L4_ERR);
 
 		if (s->flags & SF_CURR_SESS) {
 			s->flags &= ~SF_CURR_SESS;
@@ -2317,6 +2318,8 @@ void back_handle_st_cer(struct stream *s)
 		 */
 
 		int delay = 1000;
+		const int reused = (s->flags & SF_SRV_REUSED) &&
+		                   !(s->flags & SF_SRV_REUSED_ANTICIPATED);
 
 		if (s->be->timeout.connect && s->be->timeout.connect < delay)
 			delay = s->be->timeout.connect;
@@ -2327,7 +2330,7 @@ void back_handle_st_cer(struct stream *s)
 		/* only wait when we're retrying on the same server */
 		if ((si->state == SI_ST_ASS ||
 		     (s->be->lbprm.algo & BE_LB_KIND) != BE_LB_KIND_RR ||
-		     (s->be->srv_act <= 1)) && !(s->flags & SF_SRV_REUSED)) {
+		     (s->be->srv_act <= 1)) && !reused) {
 			si->state = SI_ST_TAR;
 			si->exp = tick_add(now_ms, MS_TO_TICKS(delay));
 		}
