@@ -784,7 +784,7 @@ int http_calc_maxage(struct stream *s, struct cache *cache, int *true_maxage)
 			chunk_strncat(chk, "", 1);
 			offset = (*chk->area == '"') ? 1 : 0;
 			smaxage = strtol(chk->area + offset, &endptr, 10);
-			if (unlikely(smaxage < 0 || endptr == chk->area))
+			if (unlikely(smaxage < 0 || endptr == chk->area + offset))
 				return -1;
 		}
 
@@ -796,7 +796,7 @@ int http_calc_maxage(struct stream *s, struct cache *cache, int *true_maxage)
 			chunk_strncat(chk, "", 1);
 			offset = (*chk->area == '"') ? 1 : 0;
 			maxage = strtol(chk->area + offset, &endptr, 10);
-			if (unlikely(maxage < 0 || endptr == chk->area))
+			if (unlikely(maxage < 0 || endptr == chk->area + offset))
 				return -1;
 		}
 	}
@@ -2280,6 +2280,19 @@ static int accept_encoding_normalizer(struct htx *htx, struct ist hdr_name,
 	/* Iterate over all the ACCEPT_ENCODING_MAX_ENTRIES first accept-encoding
 	 * values that might span acrosse multiple accept-encoding headers. */
 	while (http_find_header(htx, hdr_name, &ctx, 0) && count < ACCEPT_ENCODING_MAX_ENTRIES) {
+		count++;
+
+		/* As per RFC7231#5.3.4, "An Accept-Encoding header field with a
+		 * combined field-value that is empty implies that the user agent
+		 * does not want any content-coding in response."
+		 *
+		 * We must (and did) count the existence of this empty header to not
+		 * hit the `count == 0` case below, but must ignore the value to not
+		 * include VARY_ENCODING_OTHER into the final bitmap.
+		 */
+		if (istlen(ctx.value) == 0)
+			continue;
+
 		/* Turn accept-encoding value to lower case */
 		ist2bin_lc(istptr(ctx.value), ctx.value);
 
@@ -2294,8 +2307,6 @@ static int accept_encoding_normalizer(struct htx *htx, struct ist hdr_name,
 			/* Unknown encoding */
 			encoding_bitmap |= VARY_ENCODING_OTHER;
 		}
-
-		count++;
 	}
 
 	/* If a "*" was found in the accepted encodings (without a null weight),
@@ -2579,8 +2590,7 @@ static int cli_io_handler_show_cache(struct appctx *appctx)
 		while (1) {
 
 			shctx_lock(shctx_ptr(cache));
-			if (!node || (node = eb32_next_dup(node)) == NULL)
-				node = eb32_lookup_ge(&cache->entries, next_key);
+			node = eb32_lookup_ge(&cache->entries, next_key);
 			if (!node) {
 				shctx_unlock(shctx_ptr(cache));
 				appctx->ctx.cli.i0 = 0;
