@@ -25,6 +25,8 @@
 #include <haproxy/stream.h>
 #include <haproxy/tools.h>
 
+#define COMP_STATE_PROCESSING 0x01
+
 const char *http_comp_flt_id = "compression filter";
 
 struct flt_ops comp_ops;
@@ -32,6 +34,7 @@ struct flt_ops comp_ops;
 struct comp_state {
 	struct comp_ctx  *comp_ctx;   /* compression context */
 	struct comp_algo *comp_algo;  /* compression algorithm if not NULL */
+	unsigned int      flags;      /* COMP_STATE_* */
 };
 
 /* Pools used to allocate comp_state structs */
@@ -93,6 +96,7 @@ comp_strm_init(struct stream *s, struct filter *filter)
 
 	st->comp_algo = NULL;
 	st->comp_ctx  = NULL;
+	st->flags     = 0;
 	filter->ctx   = st;
 
 	/* Register post-analyzer on AN_RES_WAIT_HTTP because we need to
@@ -135,6 +139,7 @@ comp_http_headers(struct stream *s, struct filter *filter, struct http_msg *msg)
 			if (!set_compression_response_header(st, s, msg))
 				goto end;
 			register_data_filter(s, msg->chn, filter);
+			st->flags |= COMP_STATE_PROCESSING;
 		}
 	}
 
@@ -181,9 +186,9 @@ comp_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg,
 
 		next = htx_get_next_blk(htx, blk);
 		while (next && htx_get_blk_type(next) == HTX_BLK_UNUSED)
-			next = htx_get_next_blk(htx, blk);
+			next = htx_get_next_blk(htx, next);
 
-		if (!(msg->flags & HTTP_MSGF_COMPRESSING))
+		if (!(st->flags & COMP_STATE_PROCESSING))
 			goto consume;
 
 		if (htx_compression_buffer_init(htx, &trash) < 0) {
@@ -209,14 +214,16 @@ comp_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg,
 
 				if (ret == sz && !b_data(&trash))
 					next = htx_remove_blk(htx, blk);
-				else
+				else {
 					blk = htx_replace_blk_value(htx, blk, v, ist2(b_head(&trash), b_data(&trash)));
+					next = htx_get_next_blk(htx, blk);
+				}
 
 				len -= ret;
 				consumed += ret;
 				to_forward += b_data(&trash);
 				if (last)
-					msg->flags &= ~HTTP_MSGF_COMPRESSING;
+					st->flags &= ~COMP_STATE_PROCESSING;
 				break;
 
 			case HTX_BLK_TLR:
@@ -230,9 +237,10 @@ comp_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg,
 					blk = htx_get_next_blk(htx, last);
 					if (!blk)
 						goto error;
+					next = htx_get_next_blk(htx, blk);
 					to_forward += b_data(&trash);
 				}
-				msg->flags &= ~HTTP_MSGF_COMPRESSING;
+				st->flags &= ~COMP_STATE_PROCESSING;
 				/* fall through */
 
 			default:
@@ -641,11 +649,17 @@ parse_compression_options(char **args, int section, struct proxy *proxy,
 			return -1;
 		}
 		while (*(args[cur_arg])) {
-			if (comp_append_algo(comp, args[cur_arg]) < 0) {
-				memprintf(err, "'%s' : '%s' is not a supported algorithm.\n",
-					  args[0], args[cur_arg]);
+			int retval = comp_append_algo(comp, args[cur_arg]);
+			if (retval) {
+				if (retval < 0)
+					memprintf(err, "'%s' : '%s' is not a supported algorithm.\n",
+						  args[0], args[cur_arg]);
+				else
+					memprintf(err, "'%s' : out of memory while parsing algo '%s'.\n",
+						  args[0], args[cur_arg]);
 				return -1;
 			}
+
 			if (proxy->comp->algos->init(&ctx, 9) == 0)
 				proxy->comp->algos->end(&ctx);
 			else {
@@ -667,7 +681,10 @@ parse_compression_options(char **args, int section, struct proxy *proxy,
 			return -1;
 		}
 		while (*(args[cur_arg])) {
-			comp_append_type(comp, args[cur_arg]);
+			if (comp_append_type(comp, args[cur_arg])) {
+				memprintf(err, "'%s': out of memory.", args[0]);
+				return -1;
+			}
 			cur_arg++;
 			continue;
 		}
