@@ -68,8 +68,7 @@ static int alloc_raw_htx_chunk_per_thread()
 
 static void free_raw_htx_chunk_per_thread()
 {
-	free(static_raw_htx_buf);
-	static_raw_htx_buf = NULL;
+	ha_free(&static_raw_htx_buf);
 }
 
 REGISTER_PER_THREAD_ALLOC(alloc_raw_htx_chunk_per_thread);
@@ -121,7 +120,13 @@ static int get_http_auth(struct sample *smp, struct htx *htx)
 	if (chunk_initlen(&auth_method, ctx.value.ptr, 0, len) != 1)
 		return 0;
 
-	chunk_initlen(&txn->auth.method_data, p + 1, 0, ctx.value.len - len - 1);
+	/* According to RFC7235, there could be multiple spaces between the
+	 * scheme and its value, we must skip all of them.
+	 */
+	while (p < istend(ctx.value) && *p == ' ')
+		++p;
+
+	chunk_initlen(&txn->auth.method_data, p, 0, istend(ctx.value) - p);
 
 	if (!strncasecmp("Basic", auth_method.area, auth_method.data)) {
 		struct buffer *http_auth = get_trash_chunk();
@@ -209,12 +214,8 @@ struct htx *smp_prefetch_htx(struct sample *smp, struct channel *chn, struct che
 		return NULL;
 	}
 
-	if (!s->txn) {
-		if (unlikely(!http_alloc_txn(s)))
-			return NULL; /* not enough memory */
-		http_init_txn(s);
-		txn = s->txn;
-	}
+	if (!s->txn && !http_create_txn(s))
+		return NULL;
 	txn = s->txn;
 	msg = (!(chn->flags & CF_ISRESP) ? &txn->req : &txn->rsp);
 
@@ -597,7 +598,7 @@ static int smp_fetch_body(const struct arg *args, struct sample *smp, const char
 		struct htx_blk *blk = htx_get_blk(htx, pos);
 		enum htx_blk_type type = htx_get_blk_type(blk);
 
-		if (type == HTX_BLK_EOM || type == HTX_BLK_TLR || type == HTX_BLK_EOT) {
+		if (type == HTX_BLK_TLR || type == HTX_BLK_EOT) {
 			finished = 1;
 			break;
 		}
@@ -638,7 +639,7 @@ static int smp_fetch_body_len(const struct arg *args, struct sample *smp, const
 		struct htx_blk *blk = htx_get_blk(htx, pos);
 		enum htx_blk_type type = htx_get_blk_type(blk);
 
-		if (type == HTX_BLK_EOM || type == HTX_BLK_TLR || type == HTX_BLK_EOT)
+		if (type == HTX_BLK_TLR || type == HTX_BLK_EOT)
 			break;
 		if (type == HTX_BLK_DATA)
 			len += htx_get_blksz(blk);
@@ -671,7 +672,7 @@ static int smp_fetch_body_size(const struct arg *args, struct sample *smp, const
 		struct htx_blk *blk = htx_get_blk(htx, pos);
 		enum htx_blk_type type = htx_get_blk_type(blk);
 
-		if (type == HTX_BLK_EOM || type == HTX_BLK_TLR || type == HTX_BLK_EOT)
+		if (type == HTX_BLK_TLR || type == HTX_BLK_EOT)
 			break;
 		if (type == HTX_BLK_DATA)
 			len += htx_get_blksz(blk);
@@ -710,12 +711,15 @@ static int smp_fetch_url_ip(const struct arg *args, struct sample *smp, const ch
 	struct htx_sl *sl;
 	struct sockaddr_storage addr;
 
+	memset(&addr, 0, sizeof(addr));
+
 	if (!htx)
 		return 0;
 	sl = http_get_stline(htx);
-	url2sa(HTX_SL_REQ_UPTR(sl), HTX_SL_REQ_ULEN(sl), &addr, NULL);
+	if (url2sa(HTX_SL_REQ_UPTR(sl), HTX_SL_REQ_ULEN(sl), &addr, NULL) < 0)
+		return 0;
 
-	if (((struct sockaddr_in *)&addr)->sin_family != AF_INET)
+	if (addr.ss_family != AF_INET)
 		return 0;
 
 	smp->data.type = SMP_T_IPV4;
@@ -731,16 +735,19 @@ static int smp_fetch_url_port(const struct arg *args, struct sample *smp, const
 	struct htx_sl *sl;
 	struct sockaddr_storage addr;
 
+	memset(&addr, 0, sizeof(addr));
+
 	if (!htx)
 		return 0;
 	sl = http_get_stline(htx);
-	url2sa(HTX_SL_REQ_UPTR(sl), HTX_SL_REQ_ULEN(sl), &addr, NULL);
+	if (url2sa(HTX_SL_REQ_UPTR(sl), HTX_SL_REQ_ULEN(sl), &addr, NULL) < 0)
+		return 0;
 
-	if (((struct sockaddr_in *)&addr)->sin_family != AF_INET)
+	if (addr.ss_family != AF_INET)
 		return 0;
 
 	smp->data.type = SMP_T_SINT;
-	smp->data.u.sint = ntohs(((struct sockaddr_in *)&addr)->sin_port);
+	smp->data.u.sint = get_host_port(&addr);
 	smp->flags = 0;
 	return 1;
 }
@@ -769,15 +776,12 @@ static int smp_fetch_fhdr(const struct arg *args, struct sample *smp, const char
 		smp->ctx.a[0] = ctx;
 	}
 
-	if (args) {
-		if (args[0].type != ARGT_STR)
-			return 0;
-		name.ptr = args[0].data.str.area;
-		name.len = args[0].data.str.data;
+	if (args[0].type != ARGT_STR)
+		return 0;
+	name = ist2(args[0].data.str.area, args[0].data.str.data);
 
-		if (args[1].type == ARGT_SINT)
-			occ = args[1].data.sint;
-	}
+	if (args[1].type == ARGT_SINT)
+		occ = args[1].data.sint;
 
 	if (!htx)
 		return 0;
@@ -819,12 +823,10 @@ static int smp_fetch_fhdr_cnt(const struct arg *args, struct sample *smp, const
 	if (!htx)
 		return 0;
 
-	if (args && args->type == ARGT_STR) {
-		name.ptr = args->data.str.area;
-		name.len = args->data.str.data;
+	if (args->type == ARGT_STR) {
+		name = ist2(args->data.str.area, args->data.str.data);
 	} else {
-		name.ptr = NULL;
-		name.len = 0;
+		name = IST_NULL;
 	}
 
 	ctx.blk = NULL;
@@ -851,7 +853,7 @@ static int smp_fetch_hdr_names(const struct arg *args, struct sample *smp, const
 	if (!htx)
 		return 0;
 
-	if (args && args->type == ARGT_STR)
+	if (args->type == ARGT_STR)
 		del = *args[0].data.str.area;
 
 	temp = get_trash_chunk();
@@ -900,15 +902,12 @@ static int smp_fetch_hdr(const struct arg *args, struct sample *smp, const char
 		smp->ctx.a[0] = ctx;
 	}
 
-	if (args) {
-		if (args[0].type != ARGT_STR)
-			return 0;
-		name.ptr = args[0].data.str.area;
-		name.len = args[0].data.str.data;
+	if (args[0].type != ARGT_STR)
+		return 0;
+	name = ist2(args[0].data.str.area, args[0].data.str.data);
 
-		if (args[1].type == ARGT_SINT)
-			occ = args[1].data.sint;
-	}
+	if (args[1].type == ARGT_SINT)
+		occ = args[1].data.sint;
 
 	if (!htx)
 		return 0;
@@ -960,12 +959,10 @@ static int smp_fetch_hdr_cnt(const struct arg *args, struct sample *smp, const c
 	if (!htx)
 		return 0;
 
-	if (args && args->type == ARGT_STR) {
-		name.ptr = args->data.str.area;
-		name.len = args->data.str.data;
+	if (args->type == ARGT_STR) {
+		name = ist2(args->data.str.area, args->data.str.data);
 	} else {
-		name.ptr = NULL;
-		name.len = 0;
+		name = IST_NULL;
 	}
 
 	ctx.blk = NULL;
@@ -1055,7 +1052,7 @@ static int smp_fetch_path(const struct arg *args, struct sample *smp, const char
 	sl = http_get_stline(htx);
 	path = http_get_path(htx_sl_req_uri(sl));
 
-	if (kw[0] == 'p' && kw[4] == 'q') // pathq
+	if (kw[4] == 'q' && (kw[0] == 'p' || kw[0] == 'b')) // pathq or baseq
 		path = http_get_path(htx_sl_req_uri(sl));
 	else
 		path = iststop(http_get_path(htx_sl_req_uri(sl)), '?');
@@ -1104,8 +1101,12 @@ static int smp_fetch_base(const struct arg *args, struct sample *smp, const char
 	if (isttest(path)) {
 		size_t len;
 
-		for (len = 0; len < path.len && *(path.ptr + len) != '?'; len++)
-			;
+		if (kw[4] == 'q' && kw[0] == 'b') { // baseq
+			len = path.len;
+		} else {
+			for (len = 0; len < path.len && *(path.ptr + len) != '?'; len++)
+				;
+		}
 
 		if (len && *(path.ptr) == '/')
 			chunk_memcat(temp, path.ptr, len);
@@ -1352,7 +1353,7 @@ static int smp_fetch_http_auth(const struct arg *args, struct sample *smp, const
 	struct channel *chn = SMP_REQ_CHN(smp);
 	struct htx *htx = smp_prefetch_htx(smp, chn, NULL, 1);
 
-	if (!args || args->type != ARGT_USR)
+	if (args->type != ARGT_USR)
 		return 0;
 
 	if (!htx)
@@ -1372,7 +1373,7 @@ static int smp_fetch_http_auth_grp(const struct arg *args, struct sample *smp, c
 	struct channel *chn = SMP_REQ_CHN(smp);
 	struct htx *htx = smp_prefetch_htx(smp, chn, NULL, 1);
 
-	if (!args || args->type != ARGT_USR)
+	if (args->type != ARGT_USR)
 		return 0;
 
 	if (!htx)
@@ -1407,7 +1408,7 @@ static int smp_fetch_capture_req_hdr(const struct arg *args, struct sample *smp,
 	struct proxy *fe;
 	int idx;
 
-	if (!args || args->type != ARGT_SINT)
+	if (args->type != ARGT_SINT)
 		return 0;
 
 	if (!smp->strm)
@@ -1435,7 +1436,7 @@ static int smp_fetch_capture_res_hdr(const struct arg *args, struct sample *smp,
 	struct proxy *fe;
 	int idx;
 
-	if (!args || args->type != ARGT_SINT)
+	if (args->type != ARGT_SINT)
 		return 0;
 
 	if (!smp->strm)
@@ -1536,7 +1537,7 @@ static int smp_fetch_capture_req_ver(const struct arg *args, struct sample *smp,
 		return 0;
 
 	txn = smp->strm->txn;
-	if (!txn || txn->req.msg_state >= HTTP_MSG_BODY)
+	if (!txn || txn->req.msg_state < HTTP_MSG_BODY)
 		return 0;
 
 	if (txn->req.flags & HTTP_MSGF_VER_11)
@@ -1562,7 +1563,7 @@ static int smp_fetch_capture_res_ver(const struct arg *args, struct sample *smp,
 		return 0;
 
 	txn = smp->strm->txn;
-	if (!txn || txn->rsp.msg_state >= HTTP_MSG_BODY)
+	if (!txn || txn->rsp.msg_state < HTTP_MSG_BODY)
 		return 0;
 
 	if (txn->rsp.flags & HTTP_MSGF_VER_11)
@@ -1599,7 +1600,7 @@ static int smp_fetch_cookie(const struct arg *args, struct sample *smp, const ch
 	size_t cook_l = 0;
 	int found = 0;
 
-	if (args && args->type == ARGT_STR) {
+	if (args->type == ARGT_STR) {
 		cook = args->data.str.area;
 		cook_l = args->data.str.data;
 	}
@@ -1701,7 +1702,7 @@ static int smp_fetch_cookie_cnt(const struct arg *args, struct sample *smp, cons
 	size_t cook_l = 0;
 	int cnt;
 
-	if (args && args->type == ARGT_STR){
+	if (args->type == ARGT_STR){
 		cook = args->data.str.area;
 		cook_l = args->data.str.data;
 	}
@@ -1827,8 +1828,7 @@ static int smp_fetch_url_param(const struct arg *args, struct sample *smp, const
 	const char *name;
 	int name_len;
 
-	if (!args ||
-	    (args[0].type && args[0].type != ARGT_STR) ||
+	if ((args[0].type && args[0].type != ARGT_STR) ||
 	    (args[1].type && args[1].type != ARGT_STR))
 		return 0;
 
@@ -1879,7 +1879,7 @@ static int smp_fetch_body_param(const struct arg *args, struct sample *smp, cons
 	const char *name;
 	int name_len;
 
-	if (!args || (args[0].type && args[0].type != ARGT_STR))
+	if (args[0].type && args[0].type != ARGT_STR)
 		return 0;
 
 	name = "";
@@ -1902,7 +1902,7 @@ static int smp_fetch_body_param(const struct arg *args, struct sample *smp, cons
 			struct htx_blk   *blk  = htx_get_blk(htx, pos);
 			enum htx_blk_type type = htx_get_blk_type(blk);
 
-			if (type == HTX_BLK_EOM || type == HTX_BLK_TLR || type == HTX_BLK_EOT)
+			if (type == HTX_BLK_TLR || type == HTX_BLK_EOT)
 				break;
 			if (type == HTX_BLK_DATA) {
 				if (!h1_format_htx_data(htx_get_blk_value(htx, blk), temp, 0))
@@ -2060,6 +2060,7 @@ static struct sample_fetch_kw_list sample_fetch_keywords = {ILH, {
 	{ "base",               smp_fetch_base,               0,                NULL,   SMP_T_STR,  SMP_USE_HRQHV },
 	{ "base32",             smp_fetch_base32,             0,                NULL,   SMP_T_SINT, SMP_USE_HRQHV },
 	{ "base32+src",         smp_fetch_base32_src,         0,                NULL,   SMP_T_BIN,  SMP_USE_HRQHV },
+	{ "baseq",              smp_fetch_base,               0,                NULL,   SMP_T_STR,  SMP_USE_HRQHV },
 
 	/* capture are allocated and are permanent in the stream */
 	{ "capture.req.hdr",    smp_fetch_capture_req_hdr,    ARG1(1,SINT),     NULL,   SMP_T_STR,  SMP_USE_HRQHP },
