@@ -28,8 +28,8 @@
 #include <haproxy/list.h>
 #include <haproxy/listener.h>
 #include <haproxy/log.h>
-#include <haproxy/protocol-t.h>
 #include <haproxy/protocol.h>
+#include <haproxy/proxy.h>
 #include <haproxy/sample.h>
 #include <haproxy/stream.h>
 #include <haproxy/task.h>
@@ -45,8 +45,13 @@ static struct bind_kw_list bind_keywords = {
 /* list of the temporarily limited listeners because of lack of resource */
 static struct mt_list global_listener_queue = MT_LIST_HEAD_INIT(global_listener_queue);
 static struct task *global_listener_queue_task;
-static struct task *manage_global_listener_queue(struct task *t, void *context, unsigned int state);
 
+/* listener status for stats */
+const char* li_status_st[LI_STATE_COUNT] = {
+	[LI_STATUS_WAITING] = "WAITING",
+	[LI_STATUS_OPEN]    = "OPEN",
+	[LI_STATUS_FULL]    = "FULL",
+};
 
 #if defined(USE_THREAD)
 
@@ -112,8 +117,10 @@ int accept_queue_push_mp(struct accept_queue_ring *ring, struct connection *conn
 	return 1;
 }
 
-/* proceed with accepting new connections */
-static struct task *accept_queue_process(struct task *t, void *context, unsigned int state)
+/* proceed with accepting new connections. Don't mark it static so that it appears
+ * in task dumps.
+ */
+struct task *accept_queue_process(struct task *t, void *context, unsigned int state)
 {
 	struct accept_queue_ring *ring = context;
 	struct connection *conn;
@@ -131,7 +138,7 @@ static struct task *accept_queue_process(struct task *t, void *context, unsigned
 			break;
 
 		li = __objt_listener(conn->target);
-		_HA_ATOMIC_ADD(&li->thr_conn[tid], 1);
+		_HA_ATOMIC_INC(&li->thr_conn[tid]);
 		ret = li->accept(conn);
 		if (ret <= 0) {
 			/* connection was terminated by the application */
@@ -182,6 +189,18 @@ REGISTER_CONFIG_POSTPARSER("multi-threaded accept queue", accept_queue_init);
 
 #endif // USE_THREAD
 
+/* helper to get listener status for stats */
+enum li_status get_li_status(struct listener *l)
+{
+	if (!l->maxconn || l->nbconn < l->maxconn) {
+		if (l->state == LI_LIMITED)
+			return LI_STATUS_WAITING;
+		else
+			return LI_STATUS_OPEN;
+	}
+	return LI_STATUS_FULL;
+}
+
 /* adjust the listener's state and its proxy's listener counters if needed.
  * It must be called under the listener's lock, but uses atomic ops to change
  * the proxy's counters so that the proxy lock is not needed.
@@ -194,21 +213,21 @@ void listener_set_state(struct listener *l, enum li_state st)
 		/* from state */
 		switch (l->state) {
 		case LI_NEW: /* first call */
-			_HA_ATOMIC_ADD(&px->li_all, 1);
+			_HA_ATOMIC_INC(&px->li_all);
 			break;
 		case LI_INIT:
 		case LI_ASSIGNED:
 			break;
 		case LI_PAUSED:
-			_HA_ATOMIC_SUB(&px->li_paused, 1);
+			_HA_ATOMIC_DEC(&px->li_paused);
 			break;
 		case LI_LISTEN:
-			_HA_ATOMIC_SUB(&px->li_bound, 1);
+			_HA_ATOMIC_DEC(&px->li_bound);
 			break;
 		case LI_READY:
 		case LI_FULL:
 		case LI_LIMITED:
-			_HA_ATOMIC_SUB(&px->li_ready, 1);
+			_HA_ATOMIC_DEC(&px->li_ready);
 			break;
 		}
 
@@ -220,17 +239,17 @@ void listener_set_state(struct listener *l, enum li_state st)
 			break;
 		case LI_PAUSED:
 			BUG_ON(l->rx.fd == -1);
-			_HA_ATOMIC_ADD(&px->li_paused, 1);
+			_HA_ATOMIC_INC(&px->li_paused);
 			break;
 		case LI_LISTEN:
 			BUG_ON(l->rx.fd == -1);
-			_HA_ATOMIC_ADD(&px->li_bound, 1);
+			_HA_ATOMIC_INC(&px->li_bound);
 			break;
 		case LI_READY:
 		case LI_FULL:
 		case LI_LIMITED:
 			BUG_ON(l->rx.fd == -1);
-			_HA_ATOMIC_ADD(&px->li_ready, 1);
+			_HA_ATOMIC_INC(&px->li_ready);
 			break;
 		}
 	}
@@ -334,7 +353,7 @@ void default_add_listener(struct protocol *proto, struct listener *listener)
 		return;
 	listener_set_state(listener, LI_ASSIGNED);
 	listener->rx.proto = proto;
-	LIST_ADDQ(&proto->receivers, &listener->rx.proto_list);
+	LIST_APPEND(&proto->receivers, &listener->rx.proto_list);
 	proto->nb_receivers++;
 }
 
@@ -421,7 +440,7 @@ int pause_listener(struct listener *l)
 	if (l->rx.proto->suspend)
 		ret = l->rx.proto->suspend(l);
 
-	MT_LIST_DEL(&l->wait_queue);
+	MT_LIST_DELETE(&l->wait_queue);
 
 	listener_set_state(l, LI_PAUSED);
 
@@ -455,7 +474,7 @@ int resume_listener(struct listener *l)
 	/* check that another thread didn't to the job in parallel (e.g. at the
 	 * end of listen_accept() while we'd come from dequeue_all_listeners().
 	 */
-	if (MT_LIST_ADDED(&l->wait_queue))
+	if (MT_LIST_INLIST(&l->wait_queue))
 		goto end;
 
 	if ((global.mode & (MODE_DAEMON | MODE_MWORKER)) &&
@@ -494,7 +513,7 @@ static void listener_full(struct listener *l)
 {
 	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
 	if (l->state >= LI_READY) {
-		MT_LIST_DEL(&l->wait_queue);
+		MT_LIST_DELETE(&l->wait_queue);
 		if (l->state != LI_FULL) {
 			l->rx.proto->disable(l);
 			listener_set_state(l, LI_FULL);
@@ -510,7 +529,7 @@ static void limit_listener(struct listener *l, struct mt_list *list)
 {
 	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
 	if (l->state == LI_READY) {
-		MT_LIST_TRY_ADDQ(list, &l->wait_queue);
+		MT_LIST_TRY_APPEND(list, &l->wait_queue);
 		l->rx.proto->disable(l);
 		listener_set_state(l, LI_LIMITED);
 	}
@@ -578,7 +597,7 @@ void default_unbind_listener(struct listener *listener)
  */
 void do_unbind_listener(struct listener *listener)
 {
-	MT_LIST_DEL(&listener->wait_queue);
+	MT_LIST_DELETE(&listener->wait_queue);
 
 	if (listener->rx.proto->unbind)
 		listener->rx.proto->unbind(listener);
@@ -622,8 +641,8 @@ int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
 			return 0;
 		}
 		l->obj_type = OBJ_TYPE_LISTENER;
-		LIST_ADDQ(&bc->frontend->conf.listeners, &l->by_fe);
-		LIST_ADDQ(&bc->listeners, &l->by_bind);
+		LIST_APPEND(&bc->frontend->conf.listeners, &l->by_fe);
+		LIST_APPEND(&bc->listeners, &l->by_bind);
 		l->bind_conf = bc;
 		l->rx.settings = &bc->settings;
 		l->rx.owner = l;
@@ -642,9 +661,11 @@ int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
 		if (fd != -1)
 			l->rx.flags |= RX_F_INHERITED;
 
+		l->extra_counters = NULL;
+
 		HA_RWLOCK_INIT(&l->lock);
-		_HA_ATOMIC_ADD(&jobs, 1);
-		_HA_ATOMIC_ADD(&listeners, 1);
+		_HA_ATOMIC_INC(&jobs);
+		_HA_ATOMIC_INC(&listeners);
 	}
 	return 1;
 }
@@ -660,10 +681,10 @@ void __delete_listener(struct listener *listener)
 {
 	if (listener->state == LI_ASSIGNED) {
 		listener_set_state(listener, LI_INIT);
-		LIST_DEL(&listener->rx.proto_list);
+		LIST_DELETE(&listener->rx.proto_list);
 		listener->rx.proto->nb_receivers--;
-		_HA_ATOMIC_SUB(&jobs, 1);
-		_HA_ATOMIC_SUB(&listeners, 1);
+		_HA_ATOMIC_DEC(&jobs);
+		_HA_ATOMIC_DEC(&listeners);
 	}
 }
 
@@ -850,11 +871,11 @@ void listener_accept(struct listener *l)
 				goto end;
 
 			case CO_AC_RETRY: /* likely a signal */
-				_HA_ATOMIC_SUB(&l->nbconn, 1);
+				_HA_ATOMIC_DEC(&l->nbconn);
 				if (p)
-					_HA_ATOMIC_SUB(&p->feconn, 1);
+					_HA_ATOMIC_DEC(&p->feconn);
 				if (!(l->options & LI_O_UNLIMITED))
-					_HA_ATOMIC_SUB(&actconn, 1);
+					_HA_ATOMIC_DEC(&actconn);
 				continue;
 
 			case CO_AC_YIELD:
@@ -880,7 +901,7 @@ void listener_accept(struct listener *l)
 			HA_ATOMIC_UPDATE_MAX(&global.cps_max, count);
 		}
 
-		_HA_ATOMIC_ADD(&activity[tid].accepted, 1);
+		_HA_ATOMIC_INC(&activity[tid].accepted);
 
 		if (unlikely(cli_conn->handle.fd >= global.maxsock)) {
 			send_log(p, LOG_EMERG,
@@ -1005,18 +1026,18 @@ void listener_accept(struct listener *l)
 			 */
 			ring = &accept_queue_rings[t];
 			if (accept_queue_push_mp(ring, cli_conn)) {
-				_HA_ATOMIC_ADD(&activity[t].accq_pushed, 1);
+				_HA_ATOMIC_INC(&activity[t].accq_pushed);
 				tasklet_wakeup(ring->tasklet);
 				continue;
 			}
 			/* If the ring is full we do a synchronous accept on
 			 * the local thread here.
 			 */
-			_HA_ATOMIC_ADD(&activity[t].accq_full, 1);
+			_HA_ATOMIC_INC(&activity[t].accq_full);
 		}
 #endif // USE_THREAD
 
-		_HA_ATOMIC_ADD(&l->thr_conn[tid], 1);
+		_HA_ATOMIC_INC(&l->thr_conn[tid]);
 		ret = l->accept(cli_conn);
 		if (unlikely(ret <= 0)) {
 			/* The connection was closed by stream_accept(). Either
@@ -1049,13 +1070,13 @@ void listener_accept(struct listener *l)
 
  end:
 	if (next_conn)
-		_HA_ATOMIC_SUB(&l->nbconn, 1);
+		_HA_ATOMIC_DEC(&l->nbconn);
 
 	if (p && next_feconn)
-		_HA_ATOMIC_SUB(&p->feconn, 1);
+		_HA_ATOMIC_DEC(&p->feconn);
 
 	if (next_actconn)
-		_HA_ATOMIC_SUB(&actconn, 1);
+		_HA_ATOMIC_DEC(&actconn);
 
 	if ((l->state == LI_FULL && (!l->maxconn || l->nbconn < l->maxconn)) ||
 	    (l->state == LI_LIMITED &&
@@ -1113,11 +1134,11 @@ void listener_release(struct listener *l)
 	struct proxy *fe = l->bind_conf->frontend;
 
 	if (!(l->options & LI_O_UNLIMITED))
-		_HA_ATOMIC_SUB(&actconn, 1);
+		_HA_ATOMIC_DEC(&actconn);
 	if (fe)
-		_HA_ATOMIC_SUB(&fe->feconn, 1);
-	_HA_ATOMIC_SUB(&l->nbconn, 1);
-	_HA_ATOMIC_SUB(&l->thr_conn[tid], 1);
+		_HA_ATOMIC_DEC(&fe->feconn);
+	_HA_ATOMIC_DEC(&l->nbconn);
+	_HA_ATOMIC_DEC(&l->thr_conn[tid]);
 
 	if (l->state == LI_FULL || l->state == LI_LIMITED)
 		resume_listener(l);
@@ -1157,9 +1178,10 @@ REGISTER_POST_DEINIT(listener_queue_deinit);
 
 /* This is the global management task for listeners. It enables listeners waiting
  * for global resources when there are enough free resource, or at least once in
- * a while. It is designed to be called as a task.
+ * a while. It is designed to be called as a task. It's exported so that it's easy
+ * to spot in "show tasks" or "show profiling".
  */
-static struct task *manage_global_listener_queue(struct task *t, void *context, unsigned int state)
+struct task *manage_global_listener_queue(struct task *t, void *context, unsigned int state)
 {
 	/* If there are still too many concurrent connections, let's wait for
 	 * some of them to go away. We don't need to re-arm the timer because
@@ -1188,7 +1210,7 @@ static struct task *manage_global_listener_queue(struct task *t, void *context,
  */
 void bind_register_keywords(struct bind_kw_list *kwl)
 {
-	LIST_ADDQ(&bind_keywords.list, &kwl->list);
+	LIST_APPEND(&bind_keywords.list, &kwl->list);
 }
 
 /* Return a pointer to the bind keyword <kw>, or NULL if not found. If the
@@ -1250,6 +1272,37 @@ void bind_dump_kws(char **out)
 	}
 }
 
+/* Try to find in srv_keyword the word that looks closest to <word> by counting
+ * transitions between letters, digits and other characters. Will return the
+ * best matching word if found, otherwise NULL.
+ */
+const char *bind_find_best_kw(const char *word)
+{
+	uint8_t word_sig[1024];
+	uint8_t list_sig[1024];
+	const struct bind_kw_list *kwl;
+	const char *best_ptr = NULL;
+	int dist, best_dist = INT_MAX;
+	int index;
+
+	make_word_fingerprint(word_sig, word);
+	list_for_each_entry(kwl, &bind_keywords.list, list) {
+		for (index = 0; kwl->kw[index].kw != NULL; index++) {
+			make_word_fingerprint(list_sig, kwl->kw[index].kw);
+			dist = word_fingerprint_distance(word_sig, list_sig);
+			if (dist < best_dist) {
+				best_dist = dist;
+				best_ptr = kwl->kw[index].kw;
+			}
+		}
+	}
+
+	if (best_dist > 2 * strlen(word) || (best_ptr && best_dist > 2 * strlen(best_ptr)))
+		best_ptr = NULL;
+
+	return best_ptr;
+}
+
 /************************************************************************/
 /*      All supported sample and ACL keywords must be declared here.    */
 /************************************************************************/
@@ -1485,7 +1538,7 @@ static int bind_parse_proto(char **args, int cur_arg, struct proxy *px, struct b
 		return ERR_ALERT | ERR_FATAL;
 	}
 
-	proto = ist2(args[cur_arg + 1], strlen(args[cur_arg + 1]));
+	proto = ist(args[cur_arg + 1]);
 	conf->mux_proto = get_mux_proto(proto);
 	if (!conf->mux_proto) {
 		memprintf(err, "'%s' :  unknown MUX protocol '%s'", args[cur_arg], args[cur_arg+1]);
@@ -1496,7 +1549,7 @@ static int bind_parse_proto(char **args, int cur_arg, struct proxy *px, struct b
 
 /* config parser for global "tune.listener.multi-queue", accepts "on" or "off" */
 static int cfg_parse_tune_listener_mq(char **args, int section_type, struct proxy *curpx,
-                                      struct proxy *defpx, const char *file, int line,
+                                      const struct proxy *defpx, const char *file, int line,
                                       char **err)
 {
 	if (too_many_args(1, args, err, NULL))
