@@ -178,6 +178,8 @@ struct pool_head *create_pool(char *name, unsigned int size, unsigned int flags)
 		/* update per-thread pool cache if necessary */
 		for (thr = 0; thr < MAX_THREADS; thr++) {
 			LIST_INIT(&pool->cache[thr].list);
+			pool->cache[thr].tid = thr;
+			pool->cache[thr].pool = pool;
 		}
 #endif
 		HA_SPIN_INIT(&pool->lock);
@@ -267,17 +269,20 @@ void pool_free_nocache(struct pool_head *pool, void *ptr)
  * we don't want a single cache to use all the cache for itself). For this, the
  * list is scanned in reverse.
  */
-void pool_evict_from_local_cache(struct pool_head *pool)
+void pool_evict_from_local_cache(struct pool_head *pool, int full)
 {
 	struct pool_cache_head *ph = &pool->cache[tid];
 	struct pool_cache_item *item;
 
-	while (ph->count >= 16 + pool_cache_count / 8 &&
-	       pool_cache_bytes > CONFIG_HAP_POOL_CACHE_SIZE * 3 / 4) {
+	while ((ph->count && full) ||
+	       (ph->count >= 16 + pool_cache_count / 8 &&
+		pool_cache_bytes > CONFIG_HAP_POOL_CACHE_SIZE * 3 / 4)) {
 		item = LIST_NEXT(&ph->list, typeof(item), by_pool);
+		BUG_ON(&item->by_pool == &ph->list);
 		ph->count--;
 		pool_cache_bytes -= pool->size;
 		pool_cache_count--;
+		pool_check_pattern(ph, item, pool->size);
 		LIST_DELETE(&item->by_pool);
 		LIST_DELETE(&item->by_lru);
 		pool_put_to_shared_cache(pool, item);
@@ -295,11 +300,17 @@ void pool_evict_from_local_caches()
 
 	do {
 		item = LIST_PREV(&ti->pool_lru_head, struct pool_cache_item *, by_lru);
+		BUG_ON(&item->by_lru == &ti->pool_lru_head);
 		/* note: by definition we remove oldest objects so they also are the
 		 * oldest in their own pools, thus their next is the pool's head.
 		 */
 		ph = LIST_NEXT(&item->by_pool, struct pool_cache_head *, list);
+		BUG_ON(ph->tid != tid);
+
 		pool = container_of(ph - tid, struct pool_head, cache);
+		BUG_ON(pool != ph->pool);
+
+		pool_check_pattern(ph, item, pool->size);
 		LIST_DELETE(&item->by_pool);
 		LIST_DELETE(&item->by_lru);
 		ph->count--;
@@ -322,12 +333,13 @@ void pool_put_to_cache(struct pool_head *pool, void *ptr)
 	LIST_INSERT(&ph->list, &item->by_pool);
 	LIST_INSERT(&ti->pool_lru_head, &item->by_lru);
 	ph->count++;
+	pool_fill_pattern(ph, item, pool->size);
 	pool_cache_count++;
 	pool_cache_bytes += pool->size;
 
 	if (unlikely(pool_cache_bytes > CONFIG_HAP_POOL_CACHE_SIZE * 3 / 4)) {
 		if (ph->count >= 16 + pool_cache_count / 8)
-			pool_evict_from_local_cache(pool);
+			pool_evict_from_local_cache(pool, 0);
 		if (pool_cache_bytes > CONFIG_HAP_POOL_CACHE_SIZE)
 			pool_evict_from_local_caches();
 	}
@@ -466,6 +478,9 @@ void pool_gc(struct pool_head *pool_ctx)
 void *pool_destroy(struct pool_head *pool)
 {
 	if (pool) {
+#ifdef CONFIG_HAP_POOLS
+		pool_evict_from_local_cache(pool, 1);
+#endif
 		pool_flush(pool);
 		if (pool->used)
 			return pool;
