@@ -63,27 +63,40 @@ struct proto_fam proto_fam_sockpair = {
 };
 
 /* Note: must not be declared <const> as its list will be overwritten */
-static struct protocol proto_sockpair = {
-	.name = "sockpair",
-	.fam = &proto_fam_sockpair,
-	.ctrl_type = SOCK_STREAM,
-	.sock_domain = AF_CUST_SOCKPAIR,
-	.sock_type = SOCK_STREAM,
-	.sock_prot = 0,
-	.add = default_add_listener,
-	.listen = sockpair_bind_listener,
-	.enable = sockpair_enable_listener,
-	.disable = sockpair_disable_listener,
-	.unbind = default_unbind_listener,
-	.accept_conn = sockpair_accept_conn,
-	.rx_unbind = sock_unbind,
-	.rx_enable = sock_enable,
-	.rx_disable = sock_disable,
-	.rx_listening = sockpair_accepting_conn,
-	.default_iocb = &sock_accept_iocb,
-	.connect = &sockpair_connect_server,
-	.receivers = LIST_HEAD_INIT(proto_sockpair.receivers),
-	.nb_receivers = 0,
+struct protocol proto_sockpair = {
+	.name           = "sockpair",
+
+	/* connection layer */
+	.ctrl_type      = SOCK_STREAM,
+	.listen         = sockpair_bind_listener,
+	.enable         = sockpair_enable_listener,
+	.disable        = sockpair_disable_listener,
+	.add            = default_add_listener,
+	.unbind         = default_unbind_listener,
+	.accept_conn    = sockpair_accept_conn,
+	.ctrl_init      = sock_conn_ctrl_init,
+	.ctrl_close     = sock_conn_ctrl_close,
+	.connect        = sockpair_connect_server,
+	.drain          = sock_drain,
+	.check_events   = sock_check_events,
+	.ignore_events  = sock_ignore_events,
+
+	/* binding layer */
+	/* Note: suspend/resume not supported */
+
+	/* address family */
+	.fam            = &proto_fam_sockpair,
+
+	/* socket layer */
+	.sock_type      = SOCK_STREAM,
+	.sock_prot      = 0,
+	.rx_enable      = sock_enable,
+	.rx_disable     = sock_disable,
+	.rx_unbind      = sock_unbind,
+	.rx_listening   = sockpair_accepting_conn,
+	.default_iocb   = sock_accept_iocb,
+	.receivers      = LIST_HEAD_INIT(proto_sockpair.receivers),
+	.nb_receivers   = 0,
 };
 
 INITCALL1(STG_REGISTER, protocol_register, &proto_sockpair);
@@ -318,6 +331,10 @@ static int sockpair_connect_server(struct connection *conn, int flags)
 		return SF_ERR_INTERNAL;
 	}
 
+	/* if a send_proxy is there, there are data */
+	if (conn->send_proxy_ofs)
+		flags |= CONNECT_HAS_DATA;
+
 	if (global.tune.server_sndbuf)
                 setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &global.tune.server_sndbuf, sizeof(global.tune.server_sndbuf));
 
@@ -345,13 +362,7 @@ static int sockpair_connect_server(struct connection *conn, int flags)
 		conn->flags |= CO_FL_SEND_PROXY;
 
 	conn_ctrl_init(conn);       /* registers the FD */
-	fdtab[fd].linger_risk = 0;  /* no need to disable lingering */
-
-	if (conn_xprt_init(conn) < 0) {
-		conn_full_close(conn);
-		conn->flags |= CO_FL_ERROR;
-		return SF_ERR_RESOURCE;
-	}
+	HA_ATOMIC_AND(&fdtab[fd].state, ~FD_LINGER_RISK);  /* no need to disable lingering */
 
 	return SF_ERR_NONE;  /* connection is OK */
 }
@@ -462,7 +473,7 @@ struct connection *sockpair_accept_conn(struct listener *l, int *status)
 	int cfd;
 
 	if ((cfd = recv_fd_uxst(l->rx.fd)) != -1)
-		fcntl(cfd, F_SETFL, O_NONBLOCK);
+		DISGUISE(fcntl(cfd, F_SETFL, O_NONBLOCK));
 
 	if (likely(cfd != -1)) {
 		/* Perfect, the connection was accepted */
@@ -484,13 +495,13 @@ struct connection *sockpair_accept_conn(struct listener *l, int *status)
 	switch (errno) {
 	case EAGAIN:
 		ret = CO_AC_DONE; /* nothing more to accept */
-		if (fdtab[l->rx.fd].ev & (FD_POLL_HUP|FD_POLL_ERR)) {
+		if (fdtab[l->rx.fd].state & (FD_POLL_HUP|FD_POLL_ERR)) {
 			/* the listening socket might have been disabled in a shared
 			 * process and we're a collateral victim. We'll just pause for
 			 * a while in case it comes back. In the mean time, we need to
 			 * clear this sticky flag.
 			 */
-			_HA_ATOMIC_AND(&fdtab[l->rx.fd].ev, ~(FD_POLL_HUP|FD_POLL_ERR));
+			_HA_ATOMIC_AND(&fdtab[l->rx.fd].state, ~(FD_POLL_HUP|FD_POLL_ERR));
 			ret = CO_AC_PAUSE;
 		}
 		fd_cant_recv(l->rx.fd);
