@@ -39,61 +39,74 @@
 #include <haproxy/sock.h>
 #include <haproxy/sock_inet.h>
 #include <haproxy/task.h>
+#include <haproxy/tools.h>
 
 static int udp_bind_listener(struct listener *listener, char *errmsg, int errlen);
-static int udp_suspend_receiver(struct receiver *rx);
-static int udp_resume_receiver(struct receiver *rx);
 static void udp_enable_listener(struct listener *listener);
 static void udp_disable_listener(struct listener *listener);
 
 /* Note: must not be declared <const> as its list will be overwritten */
-static struct protocol proto_udp4 = {
-	.name = "udp4",
-	.fam = &proto_fam_inet4,
-	.ctrl_type = SOCK_DGRAM,
-	.sock_domain = AF_INET,
-	.sock_type = SOCK_DGRAM,
-	.sock_prot = IPPROTO_UDP,
-	.add = default_add_listener,
-	.listen = udp_bind_listener,
-	.enable = udp_enable_listener,
-	.disable = udp_disable_listener,
-	.unbind = default_unbind_listener,
-	.suspend = default_suspend_listener,
-	.resume  = default_resume_listener,
-	.rx_enable = sock_enable,
-	.rx_disable = sock_disable,
-	.rx_unbind = sock_unbind,
-	.rx_suspend = udp_suspend_receiver,
-	.rx_resume = udp_resume_receiver,
-	.receivers = LIST_HEAD_INIT(proto_udp4.receivers),
-	.nb_receivers = 0,
+struct protocol proto_udp4 = {
+	.name           = "udp4",
+
+	/* connection layer */
+	.ctrl_type      = SOCK_DGRAM,
+	.listen         = udp_bind_listener,
+	.enable         = udp_enable_listener,
+	.disable        = udp_disable_listener,
+	.add            = default_add_listener,
+	.unbind         = default_unbind_listener,
+	.suspend        = default_suspend_listener,
+	.resume         = default_resume_listener,
+
+	/* binding layer */
+	.rx_suspend     = udp_suspend_receiver,
+	.rx_resume      = udp_resume_receiver,
+
+	/* address family */
+	.fam            = &proto_fam_inet4,
+
+	/* socket layer */
+	.sock_type      = SOCK_DGRAM,
+	.sock_prot      = IPPROTO_UDP,
+	.rx_enable      = sock_enable,
+	.rx_disable     = sock_disable,
+	.rx_unbind      = sock_unbind,
+	.receivers      = LIST_HEAD_INIT(proto_udp4.receivers),
+	.nb_receivers   = 0,
 };
 
 INITCALL1(STG_REGISTER, protocol_register, &proto_udp4);
 
 /* Note: must not be declared <const> as its list will be overwritten */
-static struct protocol proto_udp6 = {
-	.name = "udp6",
-	.fam = &proto_fam_inet6,
-	.ctrl_type = SOCK_DGRAM,
-	.sock_domain = AF_INET6,
-	.sock_type = SOCK_DGRAM,
-	.sock_prot = IPPROTO_UDP,
-	.add = default_add_listener,
-	.listen = udp_bind_listener,
-	.enable = udp_enable_listener,
-	.disable = udp_disable_listener,
-	.unbind = default_unbind_listener,
-	.suspend = default_suspend_listener,
-	.resume  = default_resume_listener,
-	.rx_enable = sock_enable,
-	.rx_disable = sock_disable,
-	.rx_unbind = sock_unbind,
-	.rx_suspend = udp_suspend_receiver,
-	.rx_resume = udp_resume_receiver,
-	.receivers = LIST_HEAD_INIT(proto_udp6.receivers),
-	.nb_receivers = 0,
+struct protocol proto_udp6 = {
+	.name           = "udp6",
+
+	/* connection layer */
+	.ctrl_type      = SOCK_DGRAM,
+	.listen         = udp_bind_listener,
+	.enable         = udp_enable_listener,
+	.disable        = udp_disable_listener,
+	.add            = default_add_listener,
+	.unbind         = default_unbind_listener,
+	.suspend        = default_suspend_listener,
+	.resume         = default_resume_listener,
+
+	/* binding layer */
+	.rx_suspend     = udp_suspend_receiver,
+	.rx_resume      = udp_resume_receiver,
+
+	/* address family */
+	.fam            = &proto_fam_inet6,
+
+	/* socket layer */
+	.sock_type      = SOCK_DGRAM,
+	.sock_prot      = IPPROTO_UDP,
+	.rx_enable      = sock_enable,
+	.rx_disable     = sock_disable,
+	.rx_unbind      = sock_unbind,
+	.receivers      = LIST_HEAD_INIT(proto_udp6.receivers),
+	.nb_receivers   = 0,
 };
 
 INITCALL1(STG_REGISTER, protocol_register, &proto_udp6);
@@ -164,7 +177,7 @@ static void udp_disable_listener(struct listener *l)
  * to do this is to connect to any address that is reachable and will not be
  * used by regular traffic, and a great one is reconnecting to self.
  */
-static int udp_suspend_receiver(struct receiver *rx)
+int udp_suspend_receiver(struct receiver *rx)
 {
 	struct sockaddr_storage ss;
 	socklen_t len = sizeof(ss);
@@ -195,7 +208,7 @@ static int udp_suspend_receiver(struct receiver *rx)
  * connecting to AF_UNSPEC. The association breaks and the socket starts to
  * receive from everywhere again.
  */
-static int udp_resume_receiver(struct receiver *rx)
+int udp_resume_receiver(struct receiver *rx)
 {
 	const struct sockaddr sa = { .sa_family = AF_UNSPEC };
 
