@@ -14,12 +14,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <types/global.h>
-#include <common/config.h>
-#include <common/defaults.h>
-#include <common/regex.h>
-#include <common/standard.h>
-#include <proto/log.h>
+#include <haproxy/api.h>
+#include <haproxy/errors.h>
+#include <haproxy/global.h>
+#include <haproxy/regex.h>
+#include <haproxy/tools.h>
 
 /* regex trash buffer used by various regex tests */
 THREAD_LOCAL regmatch_t pmatch[MAX_MATCH];  /* rm_so, rm_eo for regular expressions */
@@ -58,12 +57,12 @@ int exp_replace(char *dst, unsigned int dst_size, char *src, const char *str, co
 				if (!*str)
 					return -1;
 
-				hex1 = toupper(*str++) - '0';
+				hex1 = toupper((unsigned char)*str++) - '0';
 
 				if (!*str)
 					return -1;
 
-				hex2 = toupper(*str++) - '0';
+				hex2 = toupper((unsigned char)*str++) - '0';
 
 				if (hex1 > 9) hex1 -= 'A' - '9' - 1;
 				if (hex2 > 9) hex2 -= 'A' - '9' - 1;
@@ -124,33 +123,6 @@ const char *check_replace_string(const char *str)
 }
 
 
-/* returns the pointer to an error in the replacement string, or NULL if OK */
-const char *chain_regex(struct hdr_exp **head, struct my_regex *preg,
-			int action, const char *replace, void *cond)
-{
-	struct hdr_exp *exp;
-
-	if (replace != NULL) {
-		const char *err;
-		err = check_replace_string(replace);
-		if (err)
-			return err;
-	}
-
-	while (*head != NULL)
-		head = &(*head)->next;
-
-	exp = calloc(1, sizeof(*exp));
-
-	exp->preg = preg;
-	exp->replace = replace;
-	exp->action = action;
-	exp->cond = cond;
-	*head = exp;
-
-	return NULL;
-}
-
 /* This function apply regex. It take const null terminated char as input.
  * If the function doesn't match, it returns false, else it returns true.
  * When it is compiled with JIT, this function execute strlen on the subject.
@@ -172,7 +144,7 @@ int regex_exec_match(const struct my_regex *preg, const char *subject,
 
 	/* Silently limit the number of allowed matches. max
 	 * match i the maximum value for match, in fact this
-	 * limit is not applyied.
+	 * limit is not applied.
 	 */
 
 	enmatch = nmatch;
@@ -198,7 +170,7 @@ int regex_exec_match(const struct my_regex *preg, const char *subject,
 	 */
 #ifdef USE_PCRE2
 	pm = pcre2_match_data_create_from_pattern(preg->reg, NULL);
-	ret = pcre2_match(preg->reg, (PCRE2_SPTR)subject, (PCRE2_SIZE)strlen(subject), 0, options, pm, NULL);
+	ret = preg->mfn(preg->reg, (PCRE2_SPTR)subject, (PCRE2_SIZE)strlen(subject), 0, options, pm, NULL);
 
 	if (ret < 0) {
 		pcre2_match_data_free(pm);
@@ -246,7 +218,7 @@ int regex_exec_match(const struct my_regex *preg, const char *subject,
  * <subject> can be modified during the processing. If the function doesn't
  * match, it returns false, else it returns true.
  * When it is compiled with standard POSIX regex or PCRE, this function add
- * a temporary null chracters at the end of the <subject>. The <subject> must
+ * a temporary null characters at the end of the <subject>. The <subject> must
  * have a real length of <length> + 1. Currently the only supported flag is
  * REG_NOTBOL.
  */
@@ -266,7 +238,7 @@ int regex_exec_match2(const struct my_regex *preg, char *subject, int length,
 
 	/* Silently limit the number of allowed matches. max
 	 * match i the maximum value for match, in fact this
-	 * limit is not applyied.
+	 * limit is not applied.
 	 */
 	enmatch = nmatch;
 	if (enmatch > MAX_MATCH)
@@ -280,7 +252,7 @@ int regex_exec_match2(const struct my_regex *preg, char *subject, int length,
 		options |= PCRE_NOTBOL;
 #endif
 
-	/* The value returned by pcre_exec()/pcre2_match() is one more than the highest numbered
+	/* The value returned by pcre_exec()/pcre2_(jit)_match() is one more than the highest numbered
 	 * pair that has been set. For example, if two substrings have been captured,
 	 * the returned value is 3. If there are no capturing subpatterns, the return
 	 * value from a successful match is 1, indicating that just the first pair of
@@ -291,7 +263,7 @@ int regex_exec_match2(const struct my_regex *preg, char *subject, int length,
 	 */
 #ifdef USE_PCRE2
 	pm = pcre2_match_data_create_from_pattern(preg->reg, NULL);
-	ret = pcre2_match(preg->reg, (PCRE2_SPTR)subject, (PCRE2_SIZE)length, 0, options, pm, NULL);
+	ret = preg->mfn(preg->reg, (PCRE2_SPTR)subject, (PCRE2_SIZE)length, 0, options, pm, NULL);
 
 	if (ret < 0) {
 		pcre2_match_data_free(pm);
@@ -393,6 +365,7 @@ struct my_regex *regex_comp(const char *str, int cs, int cap, char **err)
 		goto out_fail_alloc;
 	}
 
+	regex->mfn = &pcre2_match;
 #if defined(USE_PCRE2_JIT)
 	jit = pcre2_jit_compile(regex->reg, PCRE2_JIT_COMPLETE);
 	/*
@@ -403,6 +376,8 @@ struct my_regex *regex_comp(const char *str, int cs, int cap, char **err)
 		pcre2_code_free(regex->reg);
 		memprintf(err, "regex '%s' jit compilation failed", str);
 		goto out_fail_alloc;
+	} else {
+		regex->mfn = &pcre2_jit_match;
 	}
 #endif
 
