@@ -25,11 +25,13 @@
 #include <haproxy/errors.h>
 #include <haproxy/list.h>
 #include <haproxy/log.h>
+#include <haproxy/proxy.h>
 #include <haproxy/ring.h>
 #include <haproxy/signal.h>
 #include <haproxy/sink.h>
 #include <haproxy/stream_interface.h>
 #include <haproxy/time.h>
+#include <haproxy/tools.h>
 
 struct list sink_list = LIST_HEAD_INIT(sink_list);
 
@@ -77,14 +79,14 @@ static struct sink *__sink_new(const char *name, const char *desc, int fmt)
 	sink->ctx.fd = -1;
 	sink->ctx.dropped = 0;
 	HA_RWLOCK_INIT(&sink->ctx.lock);
-	LIST_ADDQ(&sink_list, &sink->sink_list);
+	LIST_APPEND(&sink_list, &sink->sink_list);
  end:
 	return sink;
 
  err:
-	free(sink->name); sink->name = NULL;
-	free(sink->desc); sink->desc = NULL;
-	free(sink); sink = NULL;
+	ha_free(&sink->name);
+	ha_free(&sink->desc);
+	ha_free(&sink);
 
 	return NULL;
 }
@@ -139,7 +141,7 @@ struct sink *sink_new_buf(const char *name, const char *desc, enum log_fmt fmt,
 
 	sink->ctx.ring = ring_new(size);
 	if (!sink->ctx.ring) {
-		LIST_DEL(&sink->sink_list);
+		LIST_DELETE(&sink->sink_list);
 		free(sink->name);
 		free(sink->desc);
 		free(sink);
@@ -357,7 +359,7 @@ static void sink_forward_io_handler(struct appctx *appctx)
 	if (unlikely(ofs == ~0)) {
 		ofs = 0;
 
-		HA_ATOMIC_ADD(b_peek(buf, ofs), 1);
+		HA_ATOMIC_INC(b_peek(buf, ofs));
 		ofs += ring->ofs;
 	}
 
@@ -371,7 +373,7 @@ static void sink_forward_io_handler(struct appctx *appctx)
 		 */
 		ofs -= ring->ofs;
 		BUG_ON(ofs >= buf->size);
-		HA_ATOMIC_SUB(b_peek(buf, ofs), 1);
+		HA_ATOMIC_DEC(b_peek(buf, ofs));
 
 		ret = 1;
 		while (ofs + 1 < b_data(buf)) {
@@ -401,7 +403,7 @@ static void sink_forward_io_handler(struct appctx *appctx)
 			ofs += cnt + msg_len;
 		}
 
-		HA_ATOMIC_ADD(b_peek(buf, ofs), 1);
+		HA_ATOMIC_INC(b_peek(buf, ofs));
 		ofs += ring->ofs;
 		sft->ofs = ofs;
 	}
@@ -410,7 +412,7 @@ static void sink_forward_io_handler(struct appctx *appctx)
 	if (ret) {
 		/* let's be woken up once new data arrive */
 		HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
-		LIST_ADDQ(&ring->waiters, &appctx->wait_entry);
+		LIST_APPEND(&ring->waiters, &appctx->wait_entry);
 		HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
 		si_rx_endp_done(si);
 	}
@@ -497,7 +499,7 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 	if (unlikely(ofs == ~0)) {
 		ofs = 0;
 
-		HA_ATOMIC_ADD(b_peek(buf, ofs), 1);
+		HA_ATOMIC_INC(b_peek(buf, ofs));
 		ofs += ring->ofs;
 	}
 
@@ -511,7 +513,7 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 		 */
 		ofs -= ring->ofs;
 		BUG_ON(ofs >= buf->size);
-		HA_ATOMIC_SUB(b_peek(buf, ofs), 1);
+		HA_ATOMIC_DEC(b_peek(buf, ofs));
 
 		ret = 1;
 		while (ofs + 1 < b_data(buf)) {
@@ -545,7 +547,7 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 			ofs += cnt + msg_len;
 		}
 
-		HA_ATOMIC_ADD(b_peek(buf, ofs), 1);
+		HA_ATOMIC_INC(b_peek(buf, ofs));
 		ofs += ring->ofs;
 		sft->ofs = ofs;
 	}
@@ -554,7 +556,7 @@ static void sink_forward_oc_io_handler(struct appctx *appctx)
 	if (ret) {
 		/* let's be woken up once new data arrive */
 		HA_RWLOCK_WRLOCK(LOGSRV_LOCK, &ring->lock);
-		LIST_ADDQ(&ring->waiters, &appctx->wait_entry);
+		LIST_APPEND(&ring->waiters, &appctx->wait_entry);
 		HA_RWLOCK_WRUNLOCK(LOGSRV_LOCK, &ring->lock);
 		si_rx_endp_done(si);
 	}
@@ -653,7 +655,7 @@ static struct appctx *sink_forward_session_create(struct sink *sink, struct sink
 		goto out_free_appctx;
 	}
 
-	if ((s = stream_new(sess, &appctx->obj_type)) == NULL) {
+	if ((s = stream_new(sess, &appctx->obj_type, &BUF_NULL)) == NULL) {
 		ha_alert("Failed to initialize stream in peer_session_create().\n");
 		goto out_free_sess;
 	}
@@ -680,7 +682,7 @@ static struct appctx *sink_forward_session_create(struct sink *sink, struct sink
 
 	/* Error unrolling */
  out_free_strm:
-	LIST_DEL(&s->list);
+	LIST_DELETE(&s->list);
 	pool_free(pool_head_stream, s);
  out_free_sess:
 	session_free(sess);
@@ -812,7 +814,8 @@ int cfg_parse_ring(const char *file, int linenum, char **args, int kwm)
 		}
 	}
 	else if (strcmp(args[0],"server") == 0) {
-		err_code |= parse_server(file, linenum, args, cfg_sink->forward_px, NULL, 1, 0, 1);
+		err_code |= parse_server(file, linenum, args, cfg_sink->forward_px, NULL,
+		                         SRV_PARSE_PARSE_ADDR|SRV_PARSE_INITIAL_RESOLVE);
 	}
 	else if (strcmp(args[0],"timeout") == 0) {
 		if (!cfg_sink || !cfg_sink->forward_px) {
@@ -917,6 +920,158 @@ err:
 	return err_code;
 }
 
+/* Creates an new sink buffer from a log server.
+ *
+ * It uses the logsrvaddress to declare a forward
+ * server for this buffer. And it initializes the
+ * forwarding.
+ *
+ * The function returns a pointer on the
+ * allocated struct sink if allocate
+ * and initialize succeed, else if it fails
+ * it returns NULL.
+ *
+ * Note: the sink is created using the name
+ *       specified into logsrv->ring_name
+ */
+struct sink *sink_new_from_logsrv(struct logsrv *logsrv)
+{
+	struct proxy *p = NULL;
+	struct sink *sink = NULL;
+	struct server *srv = NULL;
+	struct sink_forward_target *sft = NULL;
+	int i;
+
+	/* allocate new proxy to handle
+	 * forward to a stream server
+	 */
+	p = calloc(1, sizeof *p);
+	if (!p) {
+		goto error;
+        }
+
+	init_new_proxy(p);
+	sink_setup_proxy(p);
+	p->id = strdup(logsrv->ring_name);
+	p->conf.args.file = p->conf.file = strdup(logsrv->conf.file);
+	p->conf.args.line = p->conf.line = logsrv->conf.line;
+
+	/* allocate a new server to forward messages
+	 * from ring buffer
+	 */
+	srv = new_server(p);
+	if (!srv)
+		goto error;
+
+	/* init server */
+	srv->id = strdup(logsrv->ring_name);
+	srv->conf.file = strdup(logsrv->conf.file);
+	srv->conf.line = logsrv->conf.line;
+	srv->addr = logsrv->addr;
+        srv->svc_port = get_host_port(&logsrv->addr);
+	HA_SPIN_INIT(&srv->lock);
+
+	/* process per thread init */
+	srv->per_thr = calloc(global.nbthread, sizeof(*srv->per_thr));
+	if (!srv->per_thr)
+		goto error;
+
+	for (i = 0; i < global.nbthread; i++) {
+		srv->per_thr[i].idle_conns = EB_ROOT;
+		srv->per_thr[i].safe_conns = EB_ROOT;
+		srv->per_thr[i].avail_conns = EB_ROOT;
+		MT_LIST_INIT(&srv->per_thr[i].streams);
+	}
+
+	/* the servers are linked backwards
+	 * first into proxy
+	 */
+	p->srv = srv;
+	srv->next = p->srv;
+
+	/* allocate sink_forward_target descriptor */
+	sft = calloc(1, sizeof(*sft));
+	if (!sft)
+		goto error;
+
+	/* init sink_forward_target offset */
+	sft->srv = srv;
+	sft->appctx = NULL;
+	sft->ofs = ~0;
+	HA_SPIN_INIT(&sft->lock);
+
+	/* prepare description for the sink */
+	chunk_reset(&trash);
+	chunk_printf(&trash, "created from logserver declared into '%s' at line %d", logsrv->conf.file, logsrv->conf.line);
+
+	/* allocate a new sink buffer */
+	sink = sink_new_buf(logsrv->ring_name, trash.area, logsrv->format, BUFSIZE);
+	if (!sink || sink->type != SINK_TYPE_BUFFER) {
+		goto error;
+	}
+
+	/* link sink_forward_target to proxy */
+	sink->forward_px = p;
+	p->parent = sink;
+
+	/* insert into sink_forward_targets
+	 * list into sink
+	 */
+	sft->next = sink->sft;
+	sink->sft = sft;
+
+	/* mark server as an attached reader to the ring */
+	if (!ring_attach(sink->ctx.ring)) {
+		/* should never fail since there is
+		 * only one reader
+		 */
+		goto error;
+	}
+
+	/* initialize sink buffer forwarding */
+	if (!sink_init_forward(sink))
+		goto error;
+
+	/* reset familyt of logsrv to consider the ring buffer target */
+	logsrv->addr.ss_family = AF_UNSPEC;
+
+	return sink;
+error:
+	if (p) {
+		if (p->id)
+			free(p->id);
+		if (p->conf.file)
+			free(p->conf.file);
+
+		free(p);
+	}
+
+	if (srv) {
+		if (srv->id)
+			free(srv->id);
+		if (srv->conf.file)
+			free((void *)srv->conf.file);
+		if (srv->per_thr)
+		       free(srv->per_thr);
+		free(srv);
+	}
+
+	if (sft)
+		free(sft);
+
+	if (sink) {
+		if (sink->ctx.ring)
+			ring_free(sink->ctx.ring);
+
+		LIST_DELETE(&sink->sink_list);
+		free(sink->name);
+		free(sink->desc);
+		free(sink);
+	}
+
+	return NULL;
+}
+
 /*
  * Post parsing "ring" section.
  *
@@ -983,7 +1138,7 @@ int cfg_post_parse_ring()
 */
 int post_sink_resolve()
 {
-	int err_code = 0;
+	int err_code = ERR_NONE;
 	struct logsrv *logsrv, *logb;
 	struct sink *sink;
 	struct proxy *px;
@@ -991,20 +1146,61 @@ int post_sink_resolve()
 	list_for_each_entry_safe(logsrv, logb, &global.logsrvs, list) {
 		if (logsrv->type == LOG_TARGET_BUFFER) {
 			sink = sink_find(logsrv->ring_name);
-			if (!sink || sink->type != SINK_TYPE_BUFFER) {
-				ha_alert("global log server uses unknown ring named '%s'.\n", logsrv->ring_name);
+			if (!sink) {
+				/* LOG_TARGET_BUFFER but !AF_UNSPEC
+				 * means we must allocate a sink
+				 * buffer to send messages to this logsrv
+				 */
+				if (logsrv->addr.ss_family != AF_UNSPEC) {
+					sink = sink_new_from_logsrv(logsrv);
+					if (!sink) {
+						ha_alert("global stream log server declared in file '%s' at line %d cannot be initialized'.\n",
+						         logsrv->conf.file, logsrv->conf.line);
+						err_code |= ERR_ALERT | ERR_FATAL;
+					}
+				}
+				else {
+					ha_alert("global log server declared in file '%s' at line %d uses unknown ring named '%s'.\n",
+					         logsrv->conf.file, logsrv->conf.line, logsrv->ring_name);
+					err_code |= ERR_ALERT | ERR_FATAL;
+				}
+			}
+			else if (sink->type != SINK_TYPE_BUFFER) {
+				ha_alert("global log server declared in file '%s' at line %d uses incompatible ring '%s'.\n",
+				         logsrv->conf.file, logsrv->conf.line, logsrv->ring_name);
 				err_code |= ERR_ALERT | ERR_FATAL;
 			}
 			logsrv->sink = sink;
 		}
+
 	}
 
 	for (px = proxies_list; px; px = px->next) {
 		list_for_each_entry_safe(logsrv, logb, &px->logsrvs, list) {
 			if (logsrv->type == LOG_TARGET_BUFFER) {
 				sink = sink_find(logsrv->ring_name);
-				if (!sink || sink->type != SINK_TYPE_BUFFER) {
-					ha_alert("proxy '%s' log server uses unknown ring named '%s'.\n", px->id, logsrv->ring_name);
+				if (!sink) {
+					/* LOG_TARGET_BUFFER but !AF_UNSPEC
+					 * means we must allocate a sink
+					 * buffer to send messages to this logsrv
+					 */
+					if (logsrv->addr.ss_family != AF_UNSPEC) {
+						sink = sink_new_from_logsrv(logsrv);
+						if (!sink) {
+							ha_alert("log server declared in proxy section '%s' file '%s' at line %d cannot be initialized'.\n",
+							         px->id, logsrv->conf.file, logsrv->conf.line);
+							err_code |= ERR_ALERT | ERR_FATAL;
+						}
+					}
+					else {
+						ha_alert("log server declared in proxy section '%s' in file '%s' at line %d uses unknown ring named '%s'.\n",
+						         px->id, logsrv->conf.file, logsrv->conf.line, logsrv->ring_name);
+						err_code |= ERR_ALERT | ERR_FATAL;
+					}
+				}
+				else if (sink->type != SINK_TYPE_BUFFER) {
+					ha_alert("log server declared in proxy section '%s' in file '%s' at line %d uses incomatible ring named '%s'.\n",
+					         px->id, logsrv->conf.file, logsrv->conf.line, logsrv->ring_name);
 					err_code |= ERR_ALERT | ERR_FATAL;
 				}
 				logsrv->sink = sink;
@@ -1016,8 +1212,28 @@ int post_sink_resolve()
 		list_for_each_entry_safe(logsrv, logb, &px->logsrvs, list) {
 			if (logsrv->type == LOG_TARGET_BUFFER) {
 				sink = sink_find(logsrv->ring_name);
-				if (!sink || sink->type != SINK_TYPE_BUFFER) {
-					ha_alert("log-forward '%s' log server uses unknown ring named '%s'.\n", px->id, logsrv->ring_name);
+				if (!sink) {
+					/* LOG_TARGET_BUFFER but !AF_UNSPEC
+					 * means we must allocate a sink
+					 * buffer to send messages to this logsrv
+					 */
+					if (logsrv->addr.ss_family != AF_UNSPEC) {
+						sink = sink_new_from_logsrv(logsrv);
+						if (!sink) {
+							ha_alert("log server declared in log-forward section '%s' file '%s' at line %d cannot be initialized'.\n",
+							         px->id, logsrv->conf.file, logsrv->conf.line);
+							err_code |= ERR_ALERT | ERR_FATAL;
+						}
+					}
+					else {
+						ha_alert("log server declared in log-forward section '%s' in file '%s' at line %d uses unknown ring named '%s'.\n",
+							 px->id, logsrv->conf.file, logsrv->conf.line, logsrv->ring_name);
+						err_code |= ERR_ALERT | ERR_FATAL;
+					}
+				}
+				else if (sink->type != SINK_TYPE_BUFFER) {
+					ha_alert("log server declared in log-forward section '%s' in file '%s' at line %d uses unknown ring named '%s'.\n",
+						 px->id, logsrv->conf.file, logsrv->conf.line, logsrv->ring_name);
 					err_code |= ERR_ALERT | ERR_FATAL;
 				}
 				logsrv->sink = sink;
@@ -1042,7 +1258,7 @@ static void sink_deinit()
 	list_for_each_entry_safe(sink, sb, &sink_list, sink_list) {
 		if (sink->type == SINK_TYPE_BUFFER)
 			ring_free(sink->ctx.ring);
-		LIST_DEL(&sink->sink_list);
+		LIST_DELETE(&sink->sink_list);
 		free(sink->name);
 		free(sink->desc);
 		free(sink);
@@ -1053,7 +1269,7 @@ INITCALL0(STG_REGISTER, sink_init);
 REGISTER_POST_DEINIT(sink_deinit);
 
 static struct cli_kw_list cli_kws = {{ },{
-	{ { "show", "events", NULL }, "show events [<sink>] : show event sink state", cli_parse_show_events, NULL, NULL },
+	{ { "show", "events", NULL }, "show events [<sink>] [-w] [-n]          : show event sink state", cli_parse_show_events, NULL, NULL },
 	{{},}
 }};
 
