@@ -35,29 +35,20 @@ void ssl_sock_free_ssl_conf(struct ssl_bind_conf *conf)
 {
 	if (conf) {
 #if defined(OPENSSL_NPN_NEGOTIATED) && !defined(OPENSSL_NO_NEXTPROTONEG)
-		free(conf->npn_str);
-		conf->npn_str = NULL;
+		ha_free(&conf->npn_str);
 #endif
 #ifdef TLSEXT_TYPE_application_layer_protocol_negotiation
-		free(conf->alpn_str);
-		conf->alpn_str = NULL;
+		ha_free(&conf->alpn_str);
 #endif
-		free(conf->ca_file);
-		conf->ca_file = NULL;
-		free(conf->ca_verify_file);
-		conf->ca_verify_file = NULL;
-		free(conf->crl_file);
-		conf->crl_file = NULL;
-		free(conf->ciphers);
-		conf->ciphers = NULL;
-#if (HA_OPENSSL_VERSION_NUMBER >= 0x10101000L)
-		free(conf->ciphersuites);
-		conf->ciphersuites = NULL;
+		ha_free(&conf->ca_file);
+		ha_free(&conf->ca_verify_file);
+		ha_free(&conf->crl_file);
+		ha_free(&conf->ciphers);
+#ifdef HAVE_SSL_CTX_SET_CIPHERSUITES
+		ha_free(&conf->ciphersuites);
 #endif
-		free(conf->curves);
-		conf->curves = NULL;
-		free(conf->ecdhe);
-		conf->ecdhe = NULL;
+		ha_free(&conf->curves);
+		ha_free(&conf->ecdhe);
 	}
 }
 
@@ -109,7 +100,7 @@ struct ssl_bind_conf *crtlist_dup_ssl_conf(struct ssl_bind_conf *src)
 		if (!dst->ciphers)
 			goto error;
 	}
-#if (HA_OPENSSL_VERSION_NUMBER >= 0x10101000L)
+#ifdef HAVE_SSL_CTX_SET_CIPHERSUITES
 	if (src->ciphersuites) {
 		dst->ciphersuites = strdup(src->ciphersuites);
 		if (!dst->ciphersuites)
@@ -186,8 +177,8 @@ void crtlist_entry_free(struct crtlist_entry *entry)
 		return;
 
 	ebpt_delete(&entry->node);
-	LIST_DEL(&entry->by_crtlist);
-	LIST_DEL(&entry->by_ckch_store);
+	LIST_DELETE(&entry->by_crtlist);
+	LIST_DELETE(&entry->by_ckch_store);
 	crtlist_free_filters(entry->filters);
 	ssl_sock_free_ssl_conf(entry->ssl_conf);
 	free(entry->ssl_conf);
@@ -249,7 +240,7 @@ struct crtlist_entry *crtlist_entry_new()
 
 	LIST_INIT(&entry->ckch_inst);
 
-	/* initialize the nodes so we can LIST_DEL in any cases */
+	/* initialize the nodes so we can LIST_DELETE in any cases */
 	LIST_INIT(&entry->by_crtlist);
 	LIST_INIT(&entry->by_ckch_store);
 
@@ -431,8 +422,7 @@ error:
 	crtlist_free_filters(entry->filters);
 	entry->filters = NULL;
 	ssl_sock_free_ssl_conf(entry->ssl_conf);
-	free(entry->ssl_conf);
-	entry->ssl_conf = NULL;
+	ha_free(&entry->ssl_conf);
 	return cfgerr;
 }
 
@@ -550,8 +540,8 @@ int crtlist_parse_file(char *file, struct bind_conf *bind_conf, struct proxy *cu
 				entry->node.key = ckchs;
 				entry->crtlist = newlist;
 				ebpt_insert(&newlist->entries, &entry->node);
-				LIST_ADDQ(&newlist->ord_entries, &entry->by_crtlist);
-				LIST_ADDQ(&ckchs->crtlist_entry, &entry->by_ckch_store);
+				LIST_APPEND(&newlist->ord_entries, &entry->by_crtlist);
+				LIST_APPEND(&ckchs->crtlist_entry, &entry->by_ckch_store);
 
 			} else if (global_ssl.extra_files & SSL_GF_BUNDLE) {
 				/* If we didn't find the file, this could be a
@@ -598,8 +588,8 @@ int crtlist_parse_file(char *file, struct bind_conf *bind_conf, struct proxy *cu
 					entry_dup->node.key = ckchs;
 					entry_dup->crtlist = newlist;
 					ebpt_insert(&newlist->entries, &entry_dup->node);
-					LIST_ADDQ(&newlist->ord_entries, &entry_dup->by_crtlist);
-					LIST_ADDQ(&ckchs->crtlist_entry, &entry_dup->by_ckch_store);
+					LIST_APPEND(&newlist->ord_entries, &entry_dup->by_crtlist);
+					LIST_APPEND(&ckchs->crtlist_entry, &entry_dup->by_ckch_store);
 
 					entry_dup = NULL; /* the entry was used, we need a new one next round */
 				}
@@ -621,8 +611,8 @@ int crtlist_parse_file(char *file, struct bind_conf *bind_conf, struct proxy *cu
 			entry->node.key = ckchs;
 			entry->crtlist = newlist;
 			ebpt_insert(&newlist->entries, &entry->node);
-			LIST_ADDQ(&newlist->ord_entries, &entry->by_crtlist);
-			LIST_ADDQ(&ckchs->crtlist_entry, &entry->by_ckch_store);
+			LIST_APPEND(&newlist->ord_entries, &entry->by_crtlist);
+			LIST_APPEND(&ckchs->crtlist_entry, &entry->by_ckch_store);
 			found++;
 		}
 		entry = NULL;
@@ -685,7 +675,7 @@ int crtlist_load_cert_dir(char *path, struct bind_conf *bind_conf, struct crtlis
 			struct dirent *de = de_list[i];
 
 			end = strrchr(de->d_name, '.');
-			if (end && (!strcmp(end, ".issuer") || !strcmp(end, ".ocsp") || !strcmp(end, ".sctl") || !strcmp(end, ".key")))
+			if (end && (strcmp(end, ".issuer") == 0 || strcmp(end, ".ocsp") == 0 || strcmp(end, ".sctl") == 0 || strcmp(end, ".key") == 0))
 				goto ignore_entry;
 
 			snprintf(fp, sizeof(fp), "%s/%s", path, de->d_name);
@@ -716,8 +706,8 @@ int crtlist_load_cert_dir(char *path, struct bind_conf *bind_conf, struct crtlis
 			}
 			entry->node.key = ckchs;
 			entry->crtlist = dir;
-			LIST_ADDQ(&ckchs->crtlist_entry, &entry->by_ckch_store);
-			LIST_ADDQ(&dir->ord_entries, &entry->by_crtlist);
+			LIST_APPEND(&ckchs->crtlist_entry, &entry->by_ckch_store);
+			LIST_APPEND(&dir->ord_entries, &entry->by_crtlist);
 			ebpt_insert(&dir->entries, &entry->node);
 
 ignore_entry:
@@ -840,7 +830,7 @@ static void dump_crtlist_sslconf(struct buffer *buf, const struct ssl_bind_conf
 		chunk_appendf(buf, "ciphers %s", conf->ciphers);
 		space++;
 	}
-#if (HA_OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined OPENSSL_IS_BORINGSSL && !defined LIBRESSL_VERSION_NUMBER)
+#ifdef HAVE_SSL_CTX_SET_CIPHERSUITES
 	if (conf->ciphersuites) {
 		if (space) chunk_appendf(buf, " ");
 		chunk_appendf(buf, "ciphersuites %s", conf->ciphersuites);
@@ -986,7 +976,7 @@ static int cli_parse_dump_crtlist(char **args, char *payload, struct appctx *app
 	appctx->ctx.cli.p0 = NULL;
 	appctx->ctx.cli.p1 = NULL;
 
-	if (*args[3] && !strcmp(args[3], "-n")) {
+	if (*args[3] && strcmp(args[3], "-n") == 0) {
 		mode = 's';
 		filename = args[4];
 	} else {
@@ -1026,8 +1016,8 @@ static void cli_release_add_crtlist(struct appctx *appctx)
 		struct ckch_inst *inst, *inst_s;
 		/* upon error free the ckch_inst and everything inside */
 		ebpt_delete(&entry->node);
-		LIST_DEL(&entry->by_crtlist);
-		LIST_DEL(&entry->by_ckch_store);
+		LIST_DELETE(&entry->by_crtlist);
+		LIST_DELETE(&entry->by_ckch_store);
 
 		list_for_each_entry_safe(inst, inst_s, &entry->ckch_inst, by_ckchs) {
 			ckch_inst_free(inst);
@@ -1112,8 +1102,8 @@ static int cli_io_handler_add_crtlist(struct appctx *appctx)
 					/* display one dot for each new instance */
 					chunk_appendf(trash, ".");
 					i++;
-					LIST_ADDQ(&store->ckch_inst, &new_inst->by_ckchs);
-					LIST_ADDQ(&entry->ckch_inst, &new_inst->by_crtlist_entry);
+					LIST_APPEND(&store->ckch_inst, &new_inst->by_ckchs);
+					LIST_APPEND(&entry->ckch_inst, &new_inst->by_crtlist_entry);
 					new_inst->crtlist_entry = entry;
 				}
 				appctx->st2 = SETCERT_ST_INSERT;
@@ -1282,9 +1272,9 @@ static int cli_parse_add_crtlist(char **args, char *payload, struct appctx *appc
 		goto error;
 	}
 
-	LIST_ADDQ(&crtlist->ord_entries, &entry->by_crtlist);
+	LIST_APPEND(&crtlist->ord_entries, &entry->by_crtlist);
 	entry->crtlist = crtlist;
-	LIST_ADDQ(&store->crtlist_entry, &entry->by_ckch_store);
+	LIST_APPEND(&store->crtlist_entry, &entry->by_ckch_store);
 
 	appctx->st2 = SETCERT_ST_INIT;
 	appctx->ctx.cli.p0 = crtlist;
@@ -1412,8 +1402,8 @@ static int cli_parse_del_crtlist(char **args, char *payload, struct appctx *appc
 	/* upon error free the ckch_inst and everything inside */
 
 	ebpt_delete(&entry->node);
-	LIST_DEL(&entry->by_crtlist);
-	LIST_DEL(&entry->by_ckch_store);
+	LIST_DELETE(&entry->by_crtlist);
+	LIST_DELETE(&entry->by_ckch_store);
 
 	list_for_each_entry_safe(inst, inst_s, &entry->ckch_inst, by_crtlist_entry) {
 		struct sni_ctx *sni, *sni_s;
@@ -1421,12 +1411,12 @@ static int cli_parse_del_crtlist(char **args, char *payload, struct appctx *appc
 		HA_RWLOCK_WRLOCK(SNI_LOCK, &inst->bind_conf->sni_lock);
 		list_for_each_entry_safe(sni, sni_s, &inst->sni_ctx, by_ckch_inst) {
 			ebmb_delete(&sni->name);
-			LIST_DEL(&sni->by_ckch_inst);
+			LIST_DELETE(&sni->by_ckch_inst);
 			SSL_CTX_free(sni->ctx);
 			free(sni);
 		}
 		HA_RWLOCK_WRUNLOCK(SNI_LOCK, &inst->bind_conf->sni_lock);
-		LIST_DEL(&inst->by_ckchs);
+		LIST_DELETE(&inst->by_ckchs);
 		free(inst);
 	}
 
@@ -1464,9 +1454,9 @@ void crtlist_deinit()
 
 /* register cli keywords */
 static struct cli_kw_list cli_kws = {{ },{
-	{ { "add", "ssl", "crt-list", NULL }, "add ssl crt-list <filename> <certfile> [options] : add a line <certfile> to a crt-list <filename>", cli_parse_add_crtlist, cli_io_handler_add_crtlist, cli_release_add_crtlist },
-	{ { "del", "ssl", "crt-list", NULL }, "del ssl crt-list <filename> <certfile[:line]> : delete a line <certfile> in a crt-list <filename>", cli_parse_del_crtlist, NULL, NULL },
-	{ { "show", "ssl", "crt-list", NULL }, "show ssl crt-list [-n] [<filename>] : show the list of crt-lists or the content of a crt-list <filename>", cli_parse_dump_crtlist, cli_io_handler_dump_crtlist, NULL },
+	{ { "add", "ssl", "crt-list", NULL }, "add ssl crt-list <list> <cert> [opts]*  : add to crt-list file <list> a line <cert> or a payload",               cli_parse_add_crtlist, cli_io_handler_add_crtlist, cli_release_add_crtlist },
+	{ { "del", "ssl", "crt-list", NULL }, "del ssl crt-list <list> <cert[:line]>   : delete a line <cert> from crt-list file <list>",                       cli_parse_del_crtlist, NULL, NULL },
+	{ { "show", "ssl", "crt-list", NULL }, "show ssl crt-list [-n] [<list>]         : show the list of crt-lists or the content of a crt-list file <list>", cli_parse_dump_crtlist, cli_io_handler_dump_crtlist, NULL },
 	{ { NULL }, NULL, NULL, NULL } }
 };
 
