@@ -177,12 +177,12 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 
 #ifdef TCP_QUICKACK
 	/* drain is needed only to send the quick ACK */
-	conn_sock_drain(conn);
+	conn_ctrl_drain(conn);
 
 	/* re-enable quickack if it was disabled to ack all data and avoid
 	 * retransmits from the client that might trigger a real reset.
 	 */
-	setsockopt(conn->handle.fd, SOL_TCP, TCP_QUICKACK, &one, sizeof(one));
+	setsockopt(conn->handle.fd, IPPROTO_TCP, TCP_QUICKACK, &one, sizeof(one));
 #endif
 	/* lingering must absolutely be disabled so that we don't send a
 	 * shutdown(), this is critical to the TCP_REPAIR trick. When no stream
@@ -194,10 +194,10 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 	/* We're on the client-facing side, we must force to disable lingering to
 	 * ensure we will use an RST exclusively and kill any pending data.
 	 */
-	fdtab[conn->handle.fd].linger_risk = 1;
+	HA_ATOMIC_OR(&fdtab[conn->handle.fd].state, FD_LINGER_RISK);
 
 #ifdef TCP_REPAIR
-	if (setsockopt(conn->handle.fd, SOL_TCP, TCP_REPAIR, &one, sizeof(one)) == 0) {
+	if (setsockopt(conn->handle.fd, IPPROTO_TCP, TCP_REPAIR, &one, sizeof(one)) == 0) {
 		/* socket will be quiet now */
 		goto out;
 	}
@@ -208,16 +208,11 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 	 */
 #ifdef IP_TTL
 	if (conn->src && conn->src->ss_family == AF_INET)
-		setsockopt(conn->handle.fd, SOL_IP, IP_TTL, &one, sizeof(one));
+		setsockopt(conn->handle.fd, IPPROTO_IP, IP_TTL, &one, sizeof(one));
 #endif
 #ifdef IPV6_UNICAST_HOPS
-#if defined(SOL_IPV6)
-	if (conn->src && conn->src->ss_family == AF_INET6)
-		setsockopt(conn->handle.fd, SOL_IPV6, IPV6_UNICAST_HOPS, &one, sizeof(one));
-#elif defined(IPPROTO_IPV6)
 	if (conn->src && conn->src->ss_family == AF_INET6)
 		setsockopt(conn->handle.fd, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &one, sizeof(one));
-#endif
 #endif
  out:
 	/* kill the stream if any */
@@ -227,16 +222,16 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 		strm->req.analysers &= AN_REQ_FLT_END;
 		strm->res.analysers &= AN_RES_FLT_END;
 		if (strm->flags & SF_BE_ASSIGNED)
-			_HA_ATOMIC_ADD(&strm->be->be_counters.denied_req, 1);
+			_HA_ATOMIC_INC(&strm->be->be_counters.denied_req);
 		if (!(strm->flags & SF_ERR_MASK))
 			strm->flags |= SF_ERR_PRXCOND;
 		if (!(strm->flags & SF_FINST_MASK))
 			strm->flags |= SF_FINST_R;
 	}
 
-	_HA_ATOMIC_ADD(&sess->fe->fe_counters.denied_req, 1);
+	_HA_ATOMIC_INC(&sess->fe->fe_counters.denied_req);
 	if (sess->listener && sess->listener->counters)
-		_HA_ATOMIC_ADD(&sess->listener->counters->denied_req, 1);
+		_HA_ATOMIC_INC(&sess->listener->counters->denied_req);
 
 	return ACT_RET_ABRT;
 }
@@ -270,13 +265,13 @@ static enum act_parse_ret tcp_parse_set_src_dst(const char **args, int *orig_arg
 	rule->arg.expr = expr;
 	rule->action = ACT_CUSTOM;
 
-	if (!strcmp(args[*orig_arg-1], "set-src")) {
+	if (strcmp(args[*orig_arg - 1], "set-src") == 0) {
 		rule->action_ptr = tcp_action_req_set_src;
-	} else if (!strcmp(args[*orig_arg-1], "set-src-port")) {
+	} else if (strcmp(args[*orig_arg - 1], "set-src-port") == 0) {
 		rule->action_ptr = tcp_action_req_set_src_port;
-	} else if (!strcmp(args[*orig_arg-1], "set-dst")) {
+	} else if (strcmp(args[*orig_arg - 1], "set-dst") == 0) {
 		rule->action_ptr = tcp_action_req_set_dst;
-	} else if (!strcmp(args[*orig_arg-1], "set-dst-port")) {
+	} else if (strcmp(args[*orig_arg - 1], "set-dst-port") == 0) {
 		rule->action_ptr = tcp_action_req_set_dst_port;
 	} else {
 		return ACT_RET_PRS_ERR;
