@@ -261,8 +261,8 @@ static char *register_name(const char *name, int len, enum vars_scope *scope,
 		*scope = SCOPE_CHECK;
 	}
 	else {
-		memprintf(err, "invalid variable name '%s'. A variable name must be start by its scope. "
-		               "The scope can be 'proc', 'sess', 'txn', 'req', 'res' or 'check'", name);
+		memprintf(err, "invalid variable name '%.*s'. A variable name must be start by its scope. "
+		               "The scope can be 'proc', 'sess', 'txn', 'req', 'res' or 'check'", len, name);
 		return res;
 	}
 
@@ -734,7 +734,7 @@ static enum act_parse_ret parse_store(const char **args, int *arg, struct proxy
 	const char *var_name = args[*arg-1];
 	int var_len;
 	const char *kw_name;
-	int flags, set_var = 0;
+	int flags = 0, set_var = 0;
 
 	if (strncmp(var_name, "set-var", 7) == 0) {
 		var_name += 7;
@@ -774,26 +774,63 @@ static enum act_parse_ret parse_store(const char **args, int *arg, struct proxy
 
 	kw_name = args[*arg-1];
 
-	rule->arg.vars.expr = sample_parse_expr((char **)args, arg, px->conf.args.file,
-	                                        px->conf.args.line, err, &px->conf.args, NULL);
-	if (!rule->arg.vars.expr)
-		return ACT_RET_PRS_ERR;
-
 	switch (rule->from) {
-	case ACT_F_TCP_REQ_SES: flags = SMP_VAL_FE_SES_ACC; break;
-	case ACT_F_TCP_REQ_CNT: flags = SMP_VAL_FE_REQ_CNT; break;
-	case ACT_F_TCP_RES_CNT: flags = SMP_VAL_BE_RES_CNT; break;
-	case ACT_F_HTTP_REQ:    flags = SMP_VAL_FE_HRQ_HDR; break;
-	case ACT_F_HTTP_RES:    flags = SMP_VAL_BE_HRS_HDR; break;
-	case ACT_F_TCP_CHK:     flags = SMP_VAL_BE_CHK_RUL; break;
-	case ACT_F_CFG_PARSER:  flags = SMP_VAL_CFG_PARSER; break;
-	case ACT_F_CLI_PARSER:  flags = SMP_VAL_CLI_PARSER; break;
+	case ACT_F_TCP_REQ_SES:
+		flags = SMP_VAL_FE_SES_ACC;
+		px->conf.args.ctx = ARGC_TSE;
+		break;
+	case ACT_F_TCP_REQ_CNT:
+		if (px->cap & PR_CAP_FE)
+			flags |= SMP_VAL_FE_REQ_CNT;
+		if (px->cap & PR_CAP_BE)
+			flags |= SMP_VAL_BE_REQ_CNT;
+		px->conf.args.ctx = ARGC_TRQ;
+		break;
+	case ACT_F_TCP_RES_CNT:
+		if (px->cap & PR_CAP_FE)
+			flags |= SMP_VAL_FE_RES_CNT;
+		if (px->cap & PR_CAP_BE)
+			flags |= SMP_VAL_BE_RES_CNT;
+		px->conf.args.ctx = ARGC_TRS;
+		break;
+	case ACT_F_HTTP_REQ:
+		if (px->cap & PR_CAP_FE)
+			flags |= SMP_VAL_FE_HRQ_HDR;
+		if (px->cap & PR_CAP_BE)
+			flags |= SMP_VAL_BE_HRQ_HDR;
+		px->conf.args.ctx = ARGC_HRQ;
+		break;
+	case ACT_F_HTTP_RES:
+		if (px->cap & PR_CAP_FE)
+			flags |= SMP_VAL_FE_HRS_HDR;
+		if (px->cap & PR_CAP_BE)
+			flags |= SMP_VAL_BE_HRS_HDR;
+		px->conf.args.ctx =  ARGC_HRS;
+		break;
+	case ACT_F_TCP_CHK:
+		flags = SMP_VAL_BE_CHK_RUL;
+		px->conf.args.ctx = ARGC_TCK;
+		break;
+	case ACT_F_CFG_PARSER:
+		flags = SMP_VAL_CFG_PARSER;
+		px->conf.args.ctx = ARGC_CFG;
+		break;
+	case ACT_F_CLI_PARSER:
+		flags = SMP_VAL_CLI_PARSER;
+		px->conf.args.ctx = ARGC_CLI;
+		break;
 	default:
 		memprintf(err,
 			  "internal error, unexpected rule->from=%d, please report this bug!",
 			  rule->from);
 		return ACT_RET_PRS_ERR;
 	}
+
+	rule->arg.vars.expr = sample_parse_expr((char **)args, arg, px->conf.args.file,
+	                                        px->conf.args.line, err, (px->cap & PR_CAP_DEF) ? NULL: &px->conf.args, NULL);
+	if (!rule->arg.vars.expr)
+		return ACT_RET_PRS_ERR;
+
 	if (!(rule->arg.vars.expr->fetch->val & flags)) {
 		memprintf(err,
 			  "fetch method '%s' extracts information from '%s', none of which is available here",
@@ -958,7 +995,7 @@ static int vars_parse_cli_set_var(char **args, char *payload, struct appctx *app
 		goto fail;
 
 	if (rule.arg.vars.scope != SCOPE_PROC) {
-		memprintf(&err, "'%s %s': cannot set variable '%s', only scope 'proc' is permitted in the global section.", args[0], args[1], args[2]);
+		memprintf(&err, "'%s %s': cannot set variable '%s', only scope 'proc' is permitted here.", args[0], args[1], args[2]);
 		goto fail;
 	}
 
